/*
								+---------------------------------+
								|                                 |
								|    ***   OpenGL ES 2.0   ***    |
								|                                 |
								|  Copyright   -tHE SWINe- 2011  |
								|                                 |
								|           GLES20Emu.h           |
								|                                 |
								+---------------------------------+
*/

/**
 *	@file GLES20Emu.cpp
 *	@author -tHE SWINe-
 *	@brief OpenGL ES 2.0 emulation
 *	@date 2011
 *
 *	This header contains entry points of relevant OpenGL ES 2 functions / extensions.
 *		It was prepared from GLES2/gl2.h, available from <www.opengl.org>
 *
 *
 *	@date 2011-05-04
 *
 *	Added GL_AUTOCHECK functionality (note to self - sadly, this is unfeasible for
 *	OpenGL 4.0 as it requires parameter names to be present in the function prototypes,
 *	and they are regrettably missing very often).
 *
 */

/*
** Copyright (c) 2007-2009 The Khronos Group Inc.
** 
** Permission is hereby granted, free of charge, to any person obtaining a
** copy of this software and/or associated documentation files (the
** "Materials"), to deal in the Materials without restriction, including
** without limitation the rights to use, copy, modify, merge, publish,
** distribute, sublicense, and/or sell copies of the Materials, and to
** permit persons to whom the Materials are furnished to do so, subject to
** the following conditions:
** 
** The above copyright notice and this permission notice shall be included
** in all copies or substantial portions of the Materials.
** 
** THE MATERIALS ARE PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
** MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
** TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
** MATERIALS OR THE USE OR OTHER DEALINGS IN THE MATERIALS.
*/

/*
** License Applicability. Except to the extent portions of this file are
** made subject to an alternative license as permitted in the SGI Free
** Software License B, Version 1.1 (the "License"), the contents of this
** file are subject only to the provisions of the License. You may not use
** this file except in compliance with the License. You may obtain a copy
** of the License at Silicon Graphics, Inc., attn: Legal Services, 1600
** Amphitheatre Parkway, Mountain View, CA 94043-1351, or at:
** 
** http://oss.sgi.com/projects/FreeB
** 
** Note that, as provided in the License, the Software is distributed on an
** "AS IS" basis, with ALL EXPRESS AND IMPLIED WARRANTIES AND CONDITIONS
** DISCLAIMED, INCLUDING, WITHOUT LIMITATION, ANY IMPLIED WARRANTIES AND
** CONDITIONS OF MERCHANTABILITY, SATISFACTORY QUALITY, FITNESS FOR A
** PARTICULAR PURPOSE, AND NON-INFRINGEMENT.
** 
** Original Code. The Original Code is: OpenGL Sample Implementation,
** Version 1.2.1, released January 26, 2000, developed by Silicon Graphics,
** Inc. The Original Code is Copyright (c) 1991-2004 Silicon Graphics, Inc.
** Copyright in any portions created by third parties is as indicated
** elsewhere herein. All Rights Reserved.
** 
** Additional Notice Provisions: This software was created using the
** OpenGL(R) version 1.2.1 Sample Implementation published by SGI, but has
** not been independently verified as being compliant with the OpenGL(R)
** version 1.2.1 Specification.
*/

#ifndef __OPENGLES2_EMULATOR_INCLUDED
#define __OPENGLES2_EMULATOR_INCLUDED

#include <string>
#include <stdexcept>
#include <cstdarg>

namespace stl_ut {

/*
 *	bool stl_ut::Format(std::string &r_s_result, const char *p_s_fmt, ...)
 *		- equivalent of standard "c" library sprintf function,
 *		  working with std::string output
 *		- returns true on success, false on failure (not enough memory)
 *		- note this is safe against output buffer overrun, but is
 *		  still susceptible to ill-formated format string (p_s_fmt)
 *		  and bad arguments (not enough of them / not corresponding
 *		  to tags in the format string). but as the first one being run-time
 *		  error and the latter one compile-time error, this brings
 *		  enought improvement.
 */
bool Format(std::string &r_s_result, const char *p_s_fmt, ...);

}

#include <stdint.h>

#if defined(_WIN32) || defined (_WIN64)
#define WIN32_LEAND_AND_MEAN 1
#include <windows.h>
#endif // _WIN32, _WIN64

/*
 *	makes life a little bit easier when including gl.h by mistake
 */
#if defined(__gl2_h_) || defined(__GL_H__) || defined(GL_VERSION_1_1)
#error("warning: gl2.h must NOT be included in files using OpenGL30.h")
#endif

/*
 *	makes life a little bit easier when including glext.h by mistake
 */
#if defined(__glext_h_) || defined(GL_GLEXT_VERSION)
#error("glext.h must NOT be included in files using OpenGL30.h")
#endif

#define __gl2_h_
#define __gl_h_
#define __GL_H__
// does not include gl.h, defines GL up to 1.1 below, want to prevent gl.h
// header inclusion

#define __glext_h_
#define __GLEXT_H__
// glext.h is not included anymore, it does contain significant amount of
// deprecated functionality, want to prevent glext.h header inclusion

/**
 *	@def NULL_FUNCTION_POINTERS
 *	@brief if defined, initializes entry function pointers to null
 *
 *	In case NULL_FUNCTION_POINTERS is defined (default), all extension function pointers
 *		are initialized to 0, but user never needs to compare individual function values
 *		so it's posible to save some space in executable file by skipping the initialization.
 */
#define NULL_FUNCTION_POINTERS

/**
 *	@def FIX_GL_FUNCTION_LINKING_CONFLICTS
 *	@brief avoids function linking conflicts by renaming function pointers, and fixing
 *		names using #define
 *
 *	Some compilers may contain newer OpenGL libraries than this header requires
 *		(OpenGL 1.1, as supported by Microsoft Visual studio family), so names of function
 *		pointers, such as <tt>glBlendColor</tt> are present in static library, as well as in this
 *		file, causing link-time error. This is fixed by renaming the function, here
 *		<tt>glBlendColor_ptr</tt> would be used, and fixing it by macro, such as <tt>#define
 *		glBlendColor_ptr glBlendColor</tt>. The downside of this is, the Visual Studio edior
 *		will no longer show pop-up function parameters (intellisense), so this is best
 *		avoided if possible.
 */
#define FIX_GL_FUNCTION_LINKING_CONFLICTS

/**
 *	@def GL_AUTOCHECK
 *	@brief automatically checks for OpenGL errors after every OpenGL function except glGetError()
 *	@note This is only available in debug mode unless FORCE_GL_CHECK is defined.
 */
#define GL_AUTOCHECK

/**
 *	@def GL_CHECK_BLOCK
 *
 *	@param[in] p_s_file is name of file of invocation
 *	@param[in] n_line is (one-based) index of source code line of invocation
 *
 *	@brief checks for OpenGL errors and throws std::runtime_error
 *		with error descritption in case any error occured
 *	@note This is only available in debug mode unless FORCE_GL_CHECK is defined.
 */
#if defined(_DEBUG) || defined(FORCE_GL_CHECK)
#define GL_CHECK_BLOCK(p_s_file,n_line) \
	do { \
		GLenum n_error; \
		if((n_error = glGetError()) == GL_NO_ERROR) \
			break;	\
		std::string s_message; \
		switch(n_error) { \
		case GL_INVALID_ENUM: \
			stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d" \
				": OpenGL error: GL_INVALID_ENUM", p_s_file, n_line); \
			throw std::runtime_error(s_message.c_str()); \
		case GL_INVALID_VALUE: \
			stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d" \
				": OpenGL error: GL_INVALID_VALUE", p_s_file, n_line); \
			throw std::runtime_error(s_message.c_str()); \
		case GL_INVALID_OPERATION: \
			stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d" \
				": OpenGL error: GL_INVALID_OPERATION", p_s_file, n_line); \
			throw std::runtime_error(s_message.c_str()); \
		case GL_OUT_OF_MEMORY: \
			stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d" \
				": OpenGL error: GL_OUT_OF_MEMORY", p_s_file, n_line); \
			throw std::runtime_error(s_message.c_str()); \
		default: \
			stl_ut::Format(s_message, "error: file \'" "%s" "\', line %d" \
				": OpenGL error: 0x%04x", p_s_file, n_line, n_error); /* this shouldn't really happen */ \
			throw std::runtime_error(s_message.c_str()); \
		}; \
	} while(0)
#else // _DEBUG || FORCE_GL_CHECK
#define GL_CHECK_BLOCK(p_s_file,n_line) do {} while(0)
#endif // _DEBUG || FORCE_GL_CHECK

/**
 *	@brief class for handling OpenGL extensions
 *
 *	Supplies means to get OpenGL function entry point, check extension support, or get
 *	OpenGL version. Functions with name n_Get*FuncPointers() returns number of function
 *	pointers that were not found (zero is success).
 */
class CGLES20ExtensionHandler {
public:
	/**
	 *	@brief compares OpenGL version to reference
	 *
	 *	Compares supplied version to current OpenGL implementation version and returns
	 *		true if current version is greater or equal (supplied version is supported).
	 *
	 *	@param[in] n_min_major is reference minimal major version number
	 *	@param[in] n_min_minor is reference minimal minor version number
	 *
	 *	@return Returns true in case OpenGL version  is greater or equal to n_min_major.n_min_minor.
	 */
	static bool b_Support_OpenGL(int n_min_major = 3, int n_min_minor = 0);

	/**
	 *	@brief returns OpenGL version
	 *
	 *	Returns OpenGL version as (comparable) integer value.
	 *
	 *	@return Returns OpenGL version as major * 10 + minor (ie. for OpenGL 3.2 returns 32).
	 */
	static int n_OpenGL_Version();

	/**
	 *	@brief returns OpenGL version major
	 *
	 *	@return Returns OpenGL version as major (ie. for OpenGL 3.2 returns 3).
	 */
	static int n_OpenGL_Version_Major();

	/**
	 *	@brief returns OpenGL version minor
	 *
	 *	@return Returns OpenGL version as minor (ie. for OpenGL 3.2 returns 2).
	 */
	static int n_OpenGL_Version_Minor();

	/**
	 *	@brief returns OpenGL version as (major, minor) pair
	 *
	 *	@return Returns OpenGL version.
	 */
	static std::pair<int, int> t_OpenGL_Version();

	/**
	 *	@brief checks extension support
	 *
	 *	Checks support of a given OpenGL extension, returns true if extension is supported.
	 *
	 *	@param[in] p_s_extension_name is null-terminated string, containing name
	 *		of OpenGL extension encoded in UTF-8
	 *
	 *	@return Returns true in case p_s_extension_name is supported, otherwise false.
	 */
	static bool b_SupportedExtension(const char *p_s_extension_name);

	/**
	 *	@brief gets extension string
	 *
	 *	OpenGL 3.0 doesn't have classic extension string, it returns names of individual
	 *		extensions instead. This is to prevent buffer overruns, people always seem
	 *		to be copying OpenGL extensions to fixed-size buffers, causing older software
	 *		from times when extension string was shorter to fail on modern hardware.
	 *		This functions returns "classic" OpenGL extension string in std::string,
	 *		encoded in UTF-8, names of individual extensions are separated by space (' ')
	 *		character.
	 *
	 *	@return Returns true on success, false on failure (not enough memory).
	 */
	static bool Get_ExtensionString(std::string &r_s_extension_list);

	/**
	 *	@brief gets entry points of all extensions
	 *
	 *	Gets pointers of function entry points for all supported extensions and OpenGL versions.
	 *
	 *	@return Returns number of missing functions (zero means success).
	 *
	 *	@note Note this is aimed at simple and test implementations without
	 *		much need for error checking. it is always better to call individual
	 *		functions, because in case this fails, there's no telling
	 *		which extension exactly failed to initialize.
	 */
	static int n_GetAllSupportedExtensionsFuncPointers();

	/**
	 *	@brief gets entry points of all OpenGL ES 2.0 core functions
	 *
	 *	@return Returns number of missing functions (zero means success).
	 */
	static int n_GetGLES20FuncPointers();

	/**
	 *	@brief gets entry points of GL_ARB_vertex_array_object extension functions
	 *
	 *	@return Returns number of missing functions (zero means success).
	 */
	static int n_GetARBVertexArrayObjectFuncPointers();
};

#ifdef FIX_GL_FUNCTION_LINKING_CONFLICTS
#define __ConCat__(a,b) a##b
#define GLFuncPtrName(f) __ConCat__(f,_ptr)
#else //FIX_GL_FUNCTION_LINKING_CONFLICTS
#define GLFuncPtrName(f) f
#endif //FIX_GL_FUNCTION_LINKING_CONFLICTS

#include <GLES2/gl2platform.h> // ...

typedef void             GLvoid;
typedef char             GLchar;
typedef unsigned int     GLenum;
typedef unsigned char    GLboolean;
typedef unsigned int     GLbitfield;
typedef khronos_int8_t   GLbyte;
typedef short            GLshort;
typedef int              GLint;
typedef int              GLsizei;
typedef khronos_uint8_t  GLubyte;
typedef unsigned short   GLushort;
typedef unsigned int     GLuint;
typedef khronos_float_t  GLfloat;
typedef khronos_float_t  GLclampf;
typedef khronos_int32_t  GLfixed;

/* GL types for handling large vertex buffer objects */
typedef khronos_intptr_t GLintptr;
typedef khronos_ssize_t  GLsizeiptr;
// base OpenGL data types

typedef ptrdiff_t GLintptrARB;
typedef ptrdiff_t GLsizeiptrARB;
// GL_ARB_vertex_buffer_object data types (GL types for handling large vertex buffer objects)

typedef char GLcharARB;
typedef unsigned int GLhandleARB;
// GL_ARB_shader_objects data types (GL types for handling shader object handles and program/shader text)

typedef unsigned short GLhalfARB;
// GL_ARB_half_float_pixel data types ("half" precision (s10e5) float data in host memory)

typedef unsigned short GLhalfNV;
// GL_NV_half_float data types ("half" precision (s10e5) float data in host memory)

typedef int64_t GLint64EXT;
typedef uint64_t GLuint64EXT;
// GL_EXT_timer_query data types

typedef int64_t GLint64;
typedef uint64_t GLuint64;
typedef struct __GLsync *GLsync;
// GL_ARB_sync data types

typedef unsigned short GLhalf;
// GL_NV_half_float data types

#ifndef APIENTRY
#define ApiEntry
#else // APIENTRY
#define ApiEntry APIENTRY
#endif // APIENTRY
#ifndef GLStaticApi
#define GLStaticApi extern
#endif // GLStaticApi
// defines for OpenGL 1.0 / OpenGL 1.1 (staticaly linked) functions

#if defined(_WIN32) || defined (_WIN64)
#define GLApi __stdcall
#else
#define GLApi 
#endif // _WIN32, _WIN64
// defines for newer (dynamicaly linked) functions

//							    --- OpenGL ES 2.0 ---

#ifndef GL_ES_VERSION_2_0
/* OpenGL ES core versions */
#define GL_ES_VERSION_2_0 1

extern bool GLEH_OPENGL_ES_2_0;

/* ClearBufferMask */
#define GL_DEPTH_BUFFER_BIT               0x00000100
#define GL_STENCIL_BUFFER_BIT             0x00000400
#define GL_COLOR_BUFFER_BIT               0x00004000

/* Boolean */
#define GL_FALSE                          0
#define GL_TRUE                           1

/* BeginMode */
#define GL_POINTS                         0x0000
#define GL_LINES                          0x0001
#define GL_LINE_LOOP                      0x0002
#define GL_LINE_STRIP                     0x0003
#define GL_TRIANGLES                      0x0004
#define GL_TRIANGLE_STRIP                 0x0005
#define GL_TRIANGLE_FAN                   0x0006

/* AlphaFunction (not supported in ES20) */
/*      GL_NEVER */
/*      GL_LESS */
/*      GL_EQUAL */
/*      GL_LEQUAL */
/*      GL_GREATER */
/*      GL_NOTEQUAL */
/*      GL_GEQUAL */
/*      GL_ALWAYS */

/* BlendingFactorDest */
#define GL_ZERO                           0
#define GL_ONE                            1
#define GL_SRC_COLOR                      0x0300
#define GL_ONE_MINUS_SRC_COLOR            0x0301
#define GL_SRC_ALPHA                      0x0302
#define GL_ONE_MINUS_SRC_ALPHA            0x0303
#define GL_DST_ALPHA                      0x0304
#define GL_ONE_MINUS_DST_ALPHA            0x0305

/* BlendingFactorSrc */
/*      GL_ZERO */
/*      GL_ONE */
#define GL_DST_COLOR                      0x0306
#define GL_ONE_MINUS_DST_COLOR            0x0307
#define GL_SRC_ALPHA_SATURATE             0x0308
/*      GL_SRC_ALPHA */
/*      GL_ONE_MINUS_SRC_ALPHA */
/*      GL_DST_ALPHA */
/*      GL_ONE_MINUS_DST_ALPHA */

/* BlendEquationSeparate */
#define GL_FUNC_ADD                       0x8006
#define GL_BLEND_EQUATION                 0x8009
#define GL_BLEND_EQUATION_RGB             0x8009    /* same as BLEND_EQUATION */
#define GL_BLEND_EQUATION_ALPHA           0x883D

/* BlendSubtract */
#define GL_FUNC_SUBTRACT                  0x800A
#define GL_FUNC_REVERSE_SUBTRACT          0x800B

/* Separate Blend Functions */
#define GL_BLEND_DST_RGB                  0x80C8
#define GL_BLEND_SRC_RGB                  0x80C9
#define GL_BLEND_DST_ALPHA                0x80CA
#define GL_BLEND_SRC_ALPHA                0x80CB
#define GL_CONSTANT_COLOR                 0x8001
#define GL_ONE_MINUS_CONSTANT_COLOR       0x8002
#define GL_CONSTANT_ALPHA                 0x8003
#define GL_ONE_MINUS_CONSTANT_ALPHA       0x8004
#define GL_BLEND_COLOR                    0x8005

/* Buffer Objects */
#define GL_ARRAY_BUFFER                   0x8892
#define GL_ELEMENT_ARRAY_BUFFER           0x8893
#define GL_ARRAY_BUFFER_BINDING           0x8894
#define GL_ELEMENT_ARRAY_BUFFER_BINDING   0x8895

#define GL_STREAM_DRAW                    0x88E0
#define GL_STATIC_DRAW                    0x88E4
#define GL_DYNAMIC_DRAW                   0x88E8

#define GL_BUFFER_SIZE                    0x8764
#define GL_BUFFER_USAGE                   0x8765

#define GL_CURRENT_VERTEX_ATTRIB          0x8626

/* CullFaceMode */
#define GL_FRONT                          0x0404
#define GL_BACK                           0x0405
#define GL_FRONT_AND_BACK                 0x0408

/* DepthFunction */
/*      GL_NEVER */
/*      GL_LESS */
/*      GL_EQUAL */
/*      GL_LEQUAL */
/*      GL_GREATER */
/*      GL_NOTEQUAL */
/*      GL_GEQUAL */
/*      GL_ALWAYS */

/* EnableCap */
#define GL_TEXTURE_2D                     0x0DE1
#define GL_CULL_FACE                      0x0B44
#define GL_BLEND                          0x0BE2
#define GL_DITHER                         0x0BD0
#define GL_STENCIL_TEST                   0x0B90
#define GL_DEPTH_TEST                     0x0B71
#define GL_SCISSOR_TEST                   0x0C11
#define GL_POLYGON_OFFSET_FILL            0x8037
#define GL_SAMPLE_ALPHA_TO_COVERAGE       0x809E
#define GL_SAMPLE_COVERAGE                0x80A0

/* ErrorCode */
#define GL_NO_ERROR                       0
#define GL_INVALID_ENUM                   0x0500
#define GL_INVALID_VALUE                  0x0501
#define GL_INVALID_OPERATION              0x0502
#define GL_OUT_OF_MEMORY                  0x0505

/* FrontFaceDirection */
#define GL_CW                             0x0900
#define GL_CCW                            0x0901

/* GetPName */
#define GL_LINE_WIDTH                     0x0B21
#define GL_ALIASED_POINT_SIZE_RANGE       0x846D
#define GL_ALIASED_LINE_WIDTH_RANGE       0x846E
#define GL_CULL_FACE_MODE                 0x0B45
#define GL_FRONT_FACE                     0x0B46
#define GL_DEPTH_RANGE                    0x0B70
#define GL_DEPTH_WRITEMASK                0x0B72
#define GL_DEPTH_CLEAR_VALUE              0x0B73
#define GL_DEPTH_FUNC                     0x0B74
#define GL_STENCIL_CLEAR_VALUE            0x0B91
#define GL_STENCIL_FUNC                   0x0B92
#define GL_STENCIL_FAIL                   0x0B94
#define GL_STENCIL_PASS_DEPTH_FAIL        0x0B95
#define GL_STENCIL_PASS_DEPTH_PASS        0x0B96
#define GL_STENCIL_REF                    0x0B97
#define GL_STENCIL_VALUE_MASK             0x0B93
#define GL_STENCIL_WRITEMASK              0x0B98
#define GL_STENCIL_BACK_FUNC              0x8800
#define GL_STENCIL_BACK_FAIL              0x8801
#define GL_STENCIL_BACK_PASS_DEPTH_FAIL   0x8802
#define GL_STENCIL_BACK_PASS_DEPTH_PASS   0x8803
#define GL_STENCIL_BACK_REF               0x8CA3
#define GL_STENCIL_BACK_VALUE_MASK        0x8CA4
#define GL_STENCIL_BACK_WRITEMASK         0x8CA5
#define GL_VIEWPORT                       0x0BA2
#define GL_SCISSOR_BOX                    0x0C10
/*      GL_SCISSOR_TEST */
#define GL_COLOR_CLEAR_VALUE              0x0C22
#define GL_COLOR_WRITEMASK                0x0C23
#define GL_UNPACK_ALIGNMENT               0x0CF5
#define GL_PACK_ALIGNMENT                 0x0D05
#define GL_MAX_TEXTURE_SIZE               0x0D33
#define GL_MAX_VIEWPORT_DIMS              0x0D3A
#define GL_SUBPIXEL_BITS                  0x0D50
#define GL_RED_BITS                       0x0D52
#define GL_GREEN_BITS                     0x0D53
#define GL_BLUE_BITS                      0x0D54
#define GL_ALPHA_BITS                     0x0D55
#define GL_DEPTH_BITS                     0x0D56
#define GL_STENCIL_BITS                   0x0D57
#define GL_POLYGON_OFFSET_UNITS           0x2A00
/*      GL_POLYGON_OFFSET_FILL */
#define GL_POLYGON_OFFSET_FACTOR          0x8038
#define GL_TEXTURE_BINDING_2D             0x8069
#define GL_SAMPLE_BUFFERS                 0x80A8
#define GL_SAMPLES                        0x80A9
#define GL_SAMPLE_COVERAGE_VALUE          0x80AA
#define GL_SAMPLE_COVERAGE_INVERT         0x80AB

/* GetTextureParameter */
/*      GL_TEXTURE_MAG_FILTER */
/*      GL_TEXTURE_MIN_FILTER */
/*      GL_TEXTURE_WRAP_S */
/*      GL_TEXTURE_WRAP_T */

#define GL_NUM_COMPRESSED_TEXTURE_FORMATS 0x86A2
#define GL_COMPRESSED_TEXTURE_FORMATS     0x86A3

/* HintMode */
#define GL_DONT_CARE                      0x1100
#define GL_FASTEST                        0x1101
#define GL_NICEST                         0x1102

/* HintTarget */
#define GL_GENERATE_MIPMAP_HINT            0x8192

/* DataType */
#define GL_BYTE                           0x1400
#define GL_UNSIGNED_BYTE                  0x1401
#define GL_SHORT                          0x1402
#define GL_UNSIGNED_SHORT                 0x1403
#define GL_INT                            0x1404
#define GL_UNSIGNED_INT                   0x1405
#define GL_FLOAT                          0x1406
#define GL_FIXED                          0x140C

/* PixelFormat */
#define GL_DEPTH_COMPONENT                0x1902
#define GL_ALPHA                          0x1906
#define GL_RGB                            0x1907
#define GL_RGBA                           0x1908
#define GL_LUMINANCE                      0x1909
#define GL_LUMINANCE_ALPHA                0x190A

/* PixelType */
/*      GL_UNSIGNED_BYTE */
#define GL_UNSIGNED_SHORT_4_4_4_4         0x8033
#define GL_UNSIGNED_SHORT_5_5_5_1         0x8034
#define GL_UNSIGNED_SHORT_5_6_5           0x8363

/* Shaders */
#define GL_FRAGMENT_SHADER                  0x8B30
#define GL_VERTEX_SHADER                    0x8B31
#define GL_MAX_VERTEX_ATTRIBS               0x8869
#define GL_MAX_VERTEX_UNIFORM_VECTORS       0x8DFB
#define GL_MAX_VARYING_VECTORS              0x8DFC
#define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS 0x8B4D
#define GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS   0x8B4C
#define GL_MAX_TEXTURE_IMAGE_UNITS          0x8872
#define GL_MAX_FRAGMENT_UNIFORM_VECTORS     0x8DFD
#define GL_SHADER_TYPE                      0x8B4F
#define GL_DELETE_STATUS                    0x8B80
#define GL_LINK_STATUS                      0x8B82
#define GL_VALIDATE_STATUS                  0x8B83
#define GL_ATTACHED_SHADERS                 0x8B85
#define GL_ACTIVE_UNIFORMS                  0x8B86
#define GL_ACTIVE_UNIFORM_MAX_LENGTH        0x8B87
#define GL_ACTIVE_ATTRIBUTES                0x8B89
#define GL_ACTIVE_ATTRIBUTE_MAX_LENGTH      0x8B8A
#define GL_SHADING_LANGUAGE_VERSION         0x8B8C
#define GL_CURRENT_PROGRAM                  0x8B8D

/* StencilFunction */
#define GL_NEVER                          0x0200
#define GL_LESS                           0x0201
#define GL_EQUAL                          0x0202
#define GL_LEQUAL                         0x0203
#define GL_GREATER                        0x0204
#define GL_NOTEQUAL                       0x0205
#define GL_GEQUAL                         0x0206
#define GL_ALWAYS                         0x0207

/* StencilOp */
/*      GL_ZERO */
#define GL_KEEP                           0x1E00
#define GL_REPLACE                        0x1E01
#define GL_INCR                           0x1E02
#define GL_DECR                           0x1E03
#define GL_INVERT                         0x150A
#define GL_INCR_WRAP                      0x8507
#define GL_DECR_WRAP                      0x8508

/* StringName */
#define GL_VENDOR                         0x1F00
#define GL_RENDERER                       0x1F01
#define GL_VERSION                        0x1F02
#define GL_EXTENSIONS                     0x1F03

/* TextureMagFilter */
#define GL_NEAREST                        0x2600
#define GL_LINEAR                         0x2601

/* TextureMinFilter */
/*      GL_NEAREST */
/*      GL_LINEAR */
#define GL_NEAREST_MIPMAP_NEAREST         0x2700
#define GL_LINEAR_MIPMAP_NEAREST          0x2701
#define GL_NEAREST_MIPMAP_LINEAR          0x2702
#define GL_LINEAR_MIPMAP_LINEAR           0x2703

/* TextureParameterName */
#define GL_TEXTURE_MAG_FILTER             0x2800
#define GL_TEXTURE_MIN_FILTER             0x2801
#define GL_TEXTURE_WRAP_S                 0x2802
#define GL_TEXTURE_WRAP_T                 0x2803

/* TextureTarget */
/*      GL_TEXTURE_2D */
#define GL_TEXTURE                        0x1702

#define GL_TEXTURE_CUBE_MAP               0x8513
#define GL_TEXTURE_BINDING_CUBE_MAP       0x8514
#define GL_TEXTURE_CUBE_MAP_POSITIVE_X    0x8515
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_X    0x8516
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Y    0x8517
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y    0x8518
#define GL_TEXTURE_CUBE_MAP_POSITIVE_Z    0x8519
#define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z    0x851A
#define GL_MAX_CUBE_MAP_TEXTURE_SIZE      0x851C

/* TextureUnit */
#define GL_TEXTURE0                       0x84C0
#define GL_TEXTURE1                       0x84C1
#define GL_TEXTURE2                       0x84C2
#define GL_TEXTURE3                       0x84C3
#define GL_TEXTURE4                       0x84C4
#define GL_TEXTURE5                       0x84C5
#define GL_TEXTURE6                       0x84C6
#define GL_TEXTURE7                       0x84C7
#define GL_TEXTURE8                       0x84C8
#define GL_TEXTURE9                       0x84C9
#define GL_TEXTURE10                      0x84CA
#define GL_TEXTURE11                      0x84CB
#define GL_TEXTURE12                      0x84CC
#define GL_TEXTURE13                      0x84CD
#define GL_TEXTURE14                      0x84CE
#define GL_TEXTURE15                      0x84CF
#define GL_TEXTURE16                      0x84D0
#define GL_TEXTURE17                      0x84D1
#define GL_TEXTURE18                      0x84D2
#define GL_TEXTURE19                      0x84D3
#define GL_TEXTURE20                      0x84D4
#define GL_TEXTURE21                      0x84D5
#define GL_TEXTURE22                      0x84D6
#define GL_TEXTURE23                      0x84D7
#define GL_TEXTURE24                      0x84D8
#define GL_TEXTURE25                      0x84D9
#define GL_TEXTURE26                      0x84DA
#define GL_TEXTURE27                      0x84DB
#define GL_TEXTURE28                      0x84DC
#define GL_TEXTURE29                      0x84DD
#define GL_TEXTURE30                      0x84DE
#define GL_TEXTURE31                      0x84DF
#define GL_ACTIVE_TEXTURE                 0x84E0

/* TextureWrapMode */
#define GL_REPEAT                         0x2901
#define GL_CLAMP_TO_EDGE                  0x812F
#define GL_MIRRORED_REPEAT                0x8370

/* Uniform Types */
#define GL_FLOAT_VEC2                     0x8B50
#define GL_FLOAT_VEC3                     0x8B51
#define GL_FLOAT_VEC4                     0x8B52
#define GL_INT_VEC2                       0x8B53
#define GL_INT_VEC3                       0x8B54
#define GL_INT_VEC4                       0x8B55
#define GL_BOOL                           0x8B56
#define GL_BOOL_VEC2                      0x8B57
#define GL_BOOL_VEC3                      0x8B58
#define GL_BOOL_VEC4                      0x8B59
#define GL_FLOAT_MAT2                     0x8B5A
#define GL_FLOAT_MAT3                     0x8B5B
#define GL_FLOAT_MAT4                     0x8B5C
#define GL_SAMPLER_2D                     0x8B5E
#define GL_SAMPLER_CUBE                   0x8B60

/* Vertex Arrays */
#define GL_VERTEX_ATTRIB_ARRAY_ENABLED        0x8622
#define GL_VERTEX_ATTRIB_ARRAY_SIZE           0x8623
#define GL_VERTEX_ATTRIB_ARRAY_STRIDE         0x8624
#define GL_VERTEX_ATTRIB_ARRAY_TYPE           0x8625
#define GL_VERTEX_ATTRIB_ARRAY_NORMALIZED     0x886A
#define GL_VERTEX_ATTRIB_ARRAY_POINTER        0x8645
#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING 0x889F

/* Read Format */
#define GL_IMPLEMENTATION_COLOR_READ_TYPE   0x8B9A
#define GL_IMPLEMENTATION_COLOR_READ_FORMAT 0x8B9B

/* Shader Source */
#define GL_COMPILE_STATUS                 0x8B81
#define GL_INFO_LOG_LENGTH                0x8B84
#define GL_SHADER_SOURCE_LENGTH           0x8B88
#define GL_SHADER_COMPILER                0x8DFA

/* Shader Binary */
#define GL_SHADER_BINARY_FORMATS          0x8DF8
#define GL_NUM_SHADER_BINARY_FORMATS      0x8DF9

/* Shader Precision-Specified Types */
#define GL_LOW_FLOAT                      0x8DF0
#define GL_MEDIUM_FLOAT                   0x8DF1
#define GL_HIGH_FLOAT                     0x8DF2
#define GL_LOW_INT                        0x8DF3
#define GL_MEDIUM_INT                     0x8DF4
#define GL_HIGH_INT                       0x8DF5

/* Framebuffer Object. */
#define GL_FRAMEBUFFER                    0x8D40
#define GL_RENDERBUFFER                   0x8D41

#define GL_RGBA4                          0x8056
#define GL_RGB5_A1                        0x8057
#define GL_RGB565                         0x8D62
#define GL_DEPTH_COMPONENT16              0x81A5
#define GL_STENCIL_INDEX                  0x1901
#define GL_STENCIL_INDEX8                 0x8D48

#define GL_RENDERBUFFER_WIDTH             0x8D42
#define GL_RENDERBUFFER_HEIGHT            0x8D43
#define GL_RENDERBUFFER_INTERNAL_FORMAT   0x8D44
#define GL_RENDERBUFFER_RED_SIZE          0x8D50
#define GL_RENDERBUFFER_GREEN_SIZE        0x8D51
#define GL_RENDERBUFFER_BLUE_SIZE         0x8D52
#define GL_RENDERBUFFER_ALPHA_SIZE        0x8D53
#define GL_RENDERBUFFER_DEPTH_SIZE        0x8D54
#define GL_RENDERBUFFER_STENCIL_SIZE      0x8D55

#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE           0x8CD0
#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME           0x8CD1
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL         0x8CD2
#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE 0x8CD3

#define GL_COLOR_ATTACHMENT0              0x8CE0
#define GL_DEPTH_ATTACHMENT               0x8D00
#define GL_STENCIL_ATTACHMENT             0x8D20

#define GL_NONE                           0

#define GL_FRAMEBUFFER_COMPLETE                      0x8CD5
#define GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT         0x8CD6
#define GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT 0x8CD7
#define GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS         0x8CD9
#define GL_FRAMEBUFFER_UNSUPPORTED                   0x8CDD

#define GL_FRAMEBUFFER_BINDING            0x8CA6
#define GL_RENDERBUFFER_BINDING           0x8CA7
#define GL_MAX_RENDERBUFFER_SIZE          0x84E8

#define GL_INVALID_FRAMEBUFFER_OPERATION  0x0506

#ifdef __cplusplus
extern "C" {
#endif

extern void (GLApi *GLFuncPtrName(glActiveTexture))(GLenum);
extern void (GLApi *GLFuncPtrName(glAttachShader))(GLuint program, GLuint shader);
extern void (GLApi *GLFuncPtrName(glBindAttribLocation))(GLuint program, GLuint index, const GLchar *name);
extern void (GLApi *GLFuncPtrName(glBindBuffer))(GLenum, GLuint);
extern void (GLApi *GLFuncPtrName(glBindFramebuffer))(GLenum, GLuint);
extern void (GLApi *GLFuncPtrName(glBindRenderbuffer))(GLenum, GLuint);
GLStaticApi void ApiEntry         glBindTexture (GLenum target, GLuint texture);
extern void (GLApi *GLFuncPtrName(glBlendColor))(GLclampf, GLclampf, GLclampf, GLclampf);
extern void (GLApi *GLFuncPtrName(glBlendEquation))(GLenum);
extern void (GLApi *GLFuncPtrName(glBlendEquationSeparate))(GLenum modeRGB, GLenum modeAlpha);
GLStaticApi void ApiEntry         glBlendFunc (GLenum sfactor, GLenum dfactor);
extern void (GLApi *GLFuncPtrName(glBlendFuncSeparate))(GLenum, GLenum, GLenum, GLenum);
extern void (GLApi *GLFuncPtrName(glBufferData))(GLenum, GLsizeiptr, const GLvoid *, GLenum);
extern void (GLApi *GLFuncPtrName(glBufferSubData))(GLenum, GLintptr, GLsizeiptr, const GLvoid*);
extern GLenum (GLApi *GLFuncPtrName(glCheckFramebufferStatus))(GLenum);
GLStaticApi void ApiEntry         glClear (GLbitfield mask);
GLStaticApi void ApiEntry         glClearColor (GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
extern void (GLApi *GLFuncPtrName(glClearDepthf))(GLclampf);
GLStaticApi void ApiEntry         glClearStencil (GLint s);
GLStaticApi void ApiEntry         glColorMask (GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha);
extern void (GLApi *GLFuncPtrName(glCompileShader))(GLuint shader);
extern void (GLApi *GLFuncPtrName(glCompressedTexImage2D))(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, const GLvoid*);
extern void (GLApi *GLFuncPtrName(glCompressedTexSubImage2D))(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, const GLvoid*);
GLStaticApi void ApiEntry         glCopyTexImage2D (GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border);
GLStaticApi void ApiEntry         glCopyTexSubImage2D (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height);
extern GLuint (GLApi *GLFuncPtrName(glCreateProgram))(void);
extern GLuint (GLApi *GLFuncPtrName(glCreateShader))(GLenum type);
GLStaticApi void ApiEntry         glCullFace (GLenum mode);
extern void (GLApi *GLFuncPtrName(glDeleteBuffers))(GLsizei, const GLuint*);
extern void (GLApi *GLFuncPtrName(glDeleteFramebuffers))(GLsizei, const GLuint *);
extern void (GLApi *GLFuncPtrName(glDeleteProgram))(GLuint program);
extern void (GLApi *GLFuncPtrName(glDeleteRenderbuffers))(GLsizei, const GLuint *);
extern void (GLApi *GLFuncPtrName(glDeleteShader))(GLuint shader);
GLStaticApi void ApiEntry         glDeleteTextures (GLsizei n, const GLuint* textures);
GLStaticApi void ApiEntry         glDepthFunc (GLenum func);
GLStaticApi void ApiEntry         glDepthMask (GLboolean flag);
extern void (GLApi *GLFuncPtrName(glDepthRangef))(GLclampf, GLclampf);
extern void (GLApi *GLFuncPtrName(glDetachShader))(GLuint program, GLuint shader);
GLStaticApi void ApiEntry         glDisable (GLenum cap);
extern void (GLApi *GLFuncPtrName(glDisableVertexAttribArray))(GLuint index);
GLStaticApi void ApiEntry         glDrawArrays (GLenum mode, GLint first, GLsizei count);
GLStaticApi void ApiEntry         glDrawElements (GLenum mode, GLsizei count, GLenum type, const GLvoid* indices);
GLStaticApi void ApiEntry         glEnable (GLenum cap);
extern void (GLApi *GLFuncPtrName(glEnableVertexAttribArray))(GLuint index);
GLStaticApi void ApiEntry         glFinish (void);
GLStaticApi void ApiEntry         glFlush (void);
extern void (GLApi *GLFuncPtrName(glFramebufferRenderbuffer))(GLenum, GLenum, GLenum, GLuint);
extern void (GLApi *GLFuncPtrName(glFramebufferTexture2D))(GLenum, GLenum, GLenum, GLuint, GLint);
GLStaticApi void ApiEntry         glFrontFace (GLenum mode);
extern void (GLApi *GLFuncPtrName(glGenBuffers))(GLsizei, GLuint*);
extern void (GLApi *GLFuncPtrName(glGenerateMipmap))(GLenum);
extern void (GLApi *GLFuncPtrName(glGenFramebuffers))(GLsizei, GLuint *);
extern void (GLApi *GLFuncPtrName(glGenRenderbuffers))(GLsizei, GLuint *);
GLStaticApi void ApiEntry         glGenTextures (GLsizei n, GLuint* textures);
extern void (GLApi *GLFuncPtrName(glGetActiveAttrib))(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name);
extern void (GLApi *GLFuncPtrName(glGetActiveUniform))(GLuint program, GLuint index, GLsizei bufSize, GLsizei *length, GLsizei *size, GLenum *type, GLchar *name);
extern void (GLApi *GLFuncPtrName(glGetAttachedShaders))(GLuint program, GLsizei maxCount, GLsizei *count, GLuint *shaders);
extern GLint (GLApi *GLFuncPtrName(glGetAttribLocation))(GLuint program, const GLchar *name);
GLStaticApi void ApiEntry         glGetBooleanv (GLenum pname, GLboolean* params);
extern void (GLApi *GLFuncPtrName(glGetBufferParameteriv))(GLenum, GLenum, GLint*);
GLStaticApi GLenum ApiEntry glGetError (void);
GLStaticApi void ApiEntry         glGetFloatv (GLenum pname, GLfloat* params);
extern void (GLApi *GLFuncPtrName(glGetFramebufferAttachmentParameteriv))(GLenum, GLenum, GLenum, GLint *);
GLStaticApi void ApiEntry         glGetIntegerv (GLenum pname, GLint* params);
extern void (GLApi *GLFuncPtrName(glGetProgramiv))(GLuint program, GLenum pname, GLint *params);
extern void (GLApi *GLFuncPtrName(glGetProgramInfoLog))(GLuint program, GLsizei bufSize, GLsizei *length, GLchar *infoLog);
extern void (GLApi *GLFuncPtrName(glGetRenderbufferParameteriv))(GLenum, GLenum, GLint *);
extern void (GLApi *GLFuncPtrName(glGetShaderiv))(GLuint program, GLenum pname, GLint *params);
extern void (GLApi *GLFuncPtrName(glGetShaderInfoLog))(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *infoLog);
extern void (GLApi *GLFuncPtrName(glGetShaderPrecisionFormat))(GLenum, GLenum, int*, int*);
extern void (GLApi *GLFuncPtrName(glGetShaderSource))(GLuint shader, GLsizei bufSize, GLsizei *length, GLchar *source);
GLStaticApi const GLubyte* ApiEntry glGetString (GLenum name);
GLStaticApi void ApiEntry         glGetTexParameterfv (GLenum target, GLenum pname, GLfloat* params);
GLStaticApi void ApiEntry         glGetTexParameteriv (GLenum target, GLenum pname, GLint* params);
extern void (GLApi *GLFuncPtrName(glGetUniformfv))(GLuint program, GLint location, GLfloat *params);
extern void (GLApi *GLFuncPtrName(glGetUniformiv))(GLuint program, GLint location, GLint *params);
extern GLint (GLApi *GLFuncPtrName(glGetUniformLocation))(GLuint program, const GLchar *name);
extern void (GLApi *GLFuncPtrName(glGetVertexAttribfv))(GLuint index, GLenum pname, GLfloat *params);
extern void (GLApi *GLFuncPtrName(glGetVertexAttribiv))(GLuint index, GLenum pname, GLint *params);
extern void (GLApi *GLFuncPtrName(glGetVertexAttribPointerv))(GLuint index, GLenum pname, GLvoid* *pointer);
GLStaticApi void ApiEntry         glHint (GLenum target, GLenum mode);
extern GLboolean (GLApi *GLFuncPtrName(glIsBuffer))(GLuint);
GLStaticApi GLboolean ApiEntry glIsEnabled (GLenum cap);
extern GLboolean (GLApi *GLFuncPtrName(glIsFramebuffer))(GLuint);
extern GLboolean (GLApi *GLFuncPtrName(glIsProgram))(GLuint program);
extern GLboolean (GLApi *GLFuncPtrName(glIsRenderbuffer))(GLuint);
extern GLboolean (GLApi *GLFuncPtrName(glIsShader))(GLuint shader);
GLStaticApi GLboolean ApiEntry glIsTexture (GLuint texture);
GLStaticApi void ApiEntry         glLineWidth (GLfloat width);
extern void (GLApi *GLFuncPtrName(glLinkProgram))(GLuint program);
GLStaticApi void ApiEntry         glPixelStorei (GLenum pname, GLint param);
GLStaticApi void ApiEntry         glPolygonOffset (GLfloat factor, GLfloat units);
GLStaticApi void ApiEntry         glReadPixels (GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid* pixels);
extern void (GLApi *GLFuncPtrName(glReleaseShaderCompiler))();
extern void (GLApi *GLFuncPtrName(glRenderbufferStorage))(GLenum, GLenum, GLsizei, GLsizei);
extern void (GLApi *GLFuncPtrName(glSampleCoverage))(GLclampf, GLboolean);
GLStaticApi void ApiEntry         glScissor (GLint x, GLint y, GLsizei width, GLsizei height);
extern void (GLApi *GLFuncPtrName(glShaderBinary))(GLsizei, const GLuint*, GLenum, const void*, GLsizei);
extern void (GLApi *GLFuncPtrName(glShaderSource))(GLuint shader, GLsizei count, const GLchar **string, const GLint *length);
GLStaticApi void ApiEntry         glStencilFunc (GLenum func, GLint ref, GLuint mask);
extern void (GLApi *GLFuncPtrName(glStencilFuncSeparate))(GLenum face, GLenum func, GLint ref, GLuint mask);
GLStaticApi void ApiEntry         glStencilMask (GLuint mask);
extern void (GLApi *GLFuncPtrName(glStencilMaskSeparate))(GLenum face, GLuint mask);
GLStaticApi void ApiEntry         glStencilOp (GLenum fail, GLenum zfail, GLenum zpass);
extern void (GLApi *GLFuncPtrName(glStencilOpSeparate))(GLenum face, GLenum fail, GLenum zfail, GLenum zpass);
GLStaticApi void ApiEntry         glTexImage2D (GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const GLvoid* pixels);
GLStaticApi void ApiEntry         glTexParameterf (GLenum target, GLenum pname, GLfloat param);
GLStaticApi void ApiEntry         glTexParameterfv (GLenum target, GLenum pname, const GLfloat* params);
GLStaticApi void ApiEntry         glTexParameteri (GLenum target, GLenum pname, GLint param);
GLStaticApi void ApiEntry         glTexParameteriv (GLenum target, GLenum pname, const GLint* params);
GLStaticApi void ApiEntry         glTexSubImage2D (GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid* pixels);
extern void (GLApi *GLFuncPtrName(glUniform1f))(GLint location, GLfloat v0);
extern void (GLApi *GLFuncPtrName(glUniform2f))(GLint location, GLfloat v0, GLfloat v1);
extern void (GLApi *GLFuncPtrName(glUniform3f))(GLint location, GLfloat v0, GLfloat v1, GLfloat v2);
extern void (GLApi *GLFuncPtrName(glUniform4f))(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3);
extern void (GLApi *GLFuncPtrName(glUniform1i))(GLint location, GLint v0);
extern void (GLApi *GLFuncPtrName(glUniform2i))(GLint location, GLint v0, GLint v1);
extern void (GLApi *GLFuncPtrName(glUniform3i))(GLint location, GLint v0, GLint v1, GLint v2);
extern void (GLApi *GLFuncPtrName(glUniform4i))(GLint location, GLint v0, GLint v1, GLint v2, GLint v3);
extern void (GLApi *GLFuncPtrName(glUniform1fv))(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *GLFuncPtrName(glUniform2fv))(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *GLFuncPtrName(glUniform3fv))(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *GLFuncPtrName(glUniform4fv))(GLint location, GLsizei count, const GLfloat *value);
extern void (GLApi *GLFuncPtrName(glUniform1iv))(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *GLFuncPtrName(glUniform2iv))(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *GLFuncPtrName(glUniform3iv))(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *GLFuncPtrName(glUniform4iv))(GLint location, GLsizei count, const GLint *value);
extern void (GLApi *GLFuncPtrName(glUniformMatrix2fv))(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *GLFuncPtrName(glUniformMatrix3fv))(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *GLFuncPtrName(glUniformMatrix4fv))(GLint location, GLsizei count, GLboolean transpose, const GLfloat *value);
extern void (GLApi *GLFuncPtrName(glUseProgram))(GLuint program);
extern void (GLApi *GLFuncPtrName(glValidateProgram))(GLuint program);
extern void (GLApi *GLFuncPtrName(glVertexAttrib1f))(GLuint index, GLfloat x);
extern void (GLApi *GLFuncPtrName(glVertexAttrib1fv))(GLuint index, const GLfloat *v);
extern void (GLApi *GLFuncPtrName(glVertexAttrib2f))(GLuint index, GLfloat x, GLfloat y);
extern void (GLApi *GLFuncPtrName(glVertexAttrib2fv))(GLuint index, const GLfloat *v);
extern void (GLApi *GLFuncPtrName(glVertexAttrib3f))(GLuint index, GLfloat x, GLfloat y, GLfloat z);
extern void (GLApi *GLFuncPtrName(glVertexAttrib3fv))(GLuint index, const GLfloat *v);
extern void (GLApi *GLFuncPtrName(glVertexAttrib4f))(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w);
extern void (GLApi *GLFuncPtrName(glVertexAttrib4fv))(GLuint index, const GLfloat *v);
extern void (GLApi *GLFuncPtrName(glVertexAttribPointer))(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, const GLvoid *pointer);
GLStaticApi void ApiEntry         glViewport (GLint x, GLint y, GLsizei width, GLsizei height);

#if defined(GL_AUTOCHECK) && (defined(_DEBUG) || defined(FORCE_GL_CHECK))

#ifdef __cplusplus
}
#endif // __cplusplus

inline void __glachwrap_glBindTexture(GLenum target, GLuint texture, const char *p_s_file, size_t n_line) { glBindTexture(target, texture); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBlendFunc(GLenum sfactor, GLenum dfactor, const char *p_s_file, size_t n_line) { glBlendFunc(sfactor, dfactor); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glClear(GLbitfield mask, const char *p_s_file, size_t n_line) { glClear(mask); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha, const char *p_s_file, size_t n_line) { glClearColor(red, green, blue, alpha); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glClearStencil(GLint s, const char *p_s_file, size_t n_line) { glClearStencil(s); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glColorMask(GLboolean red, GLboolean green, GLboolean blue, GLboolean alpha, const char *p_s_file, size_t n_line) { glColorMask(red, green, blue, alpha); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glCopyTexImage2D(GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border, const char *p_s_file, size_t n_line) { glCopyTexImage2D(target, level, internalformat, x, y, width, height, border); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glCopyTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height, const char *p_s_file, size_t n_line) { glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glCullFace(GLenum mode, const char *p_s_file, size_t n_line) { glCullFace(mode); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDeleteTextures(GLsizei n, const GLuint* textures, const char *p_s_file, size_t n_line) { glDeleteTextures(n, textures); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDepthFunc(GLenum func, const char *p_s_file, size_t n_line) { glDepthFunc(func); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDepthMask(GLboolean flag, const char *p_s_file, size_t n_line) { glDepthMask(flag); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDisable(GLenum cap, const char *p_s_file, size_t n_line) { glDisable(cap); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDrawArrays(GLenum mode, GLint first, GLsizei count, const char *p_s_file, size_t n_line) { glDrawArrays(mode, first, count); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDrawElements(GLenum mode, GLsizei count, GLenum type, const GLvoid* indices, const char *p_s_file, size_t n_line) { glDrawElements(mode, count, type, indices); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glEnable(GLenum cap, const char *p_s_file, size_t n_line) { glEnable(cap); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glFinish(const char *p_s_file, size_t n_line) { glFinish(); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glFlush(const char *p_s_file, size_t n_line) { glFlush(); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glFrontFace(GLenum mode, const char *p_s_file, size_t n_line) { glFrontFace(mode); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGenTextures(GLsizei n, GLuint* textures, const char *p_s_file, size_t n_line) { glGenTextures(n, textures); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetBooleanv(GLenum pname, GLboolean* params, const char *p_s_file, size_t n_line) { glGetBooleanv(pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
//inline GLenum __glachwrap_glGetError(const char *p_s_file, size_t n_line) { GLenum t_result = glGetError(); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glGetFloatv(GLenum pname, GLfloat* params, const char *p_s_file, size_t n_line) { glGetFloatv(pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetIntegerv(GLenum pname, GLint* params, const char *p_s_file, size_t n_line) { glGetIntegerv(pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline const GLubyte* __glachwrap_glGetString(GLenum name, const char *p_s_file, size_t n_line) { const GLubyte* t_result = glGetString(name); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glGetTexParameterfv(GLenum target, GLenum pname, GLfloat* params, const char *p_s_file, size_t n_line) { glGetTexParameterfv(target, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetTexParameteriv(GLenum target, GLenum pname, GLint* params, const char *p_s_file, size_t n_line) { glGetTexParameteriv(target, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glHint(GLenum target, GLenum mode, const char *p_s_file, size_t n_line) { glHint(target, mode); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline GLboolean __glachwrap_glIsEnabled(GLenum cap, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsEnabled(cap); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline GLboolean __glachwrap_glIsTexture(GLuint texture, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsTexture(texture); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glLineWidth(GLfloat width, const char *p_s_file, size_t n_line) { glLineWidth(width); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glPixelStorei(GLenum pname, GLint param, const char *p_s_file, size_t n_line) { glPixelStorei(pname, param); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glPolygonOffset(GLfloat factor, GLfloat units, const char *p_s_file, size_t n_line) { glPolygonOffset(factor, units); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glReadPixels(GLint x, GLint y, GLsizei width, GLsizei height, GLenum format, GLenum type, GLvoid* pixels, const char *p_s_file, size_t n_line) { glReadPixels(x, y, width, height, format, type, pixels); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glScissor(GLint x, GLint y, GLsizei width, GLsizei height, const char *p_s_file, size_t n_line) { glScissor(x, y, width, height); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glStencilFunc(GLenum func, GLint ref, GLuint mask, const char *p_s_file, size_t n_line) { glStencilFunc(func, ref, mask); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glStencilMask(GLuint mask, const char *p_s_file, size_t n_line) { glStencilMask(mask); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glStencilOp(GLenum fail, GLenum zfail, GLenum zpass, const char *p_s_file, size_t n_line) { glStencilOp(fail, zfail, zpass); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei width, GLsizei height, GLint border, GLenum format, GLenum type, const GLvoid* pixels, const char *p_s_file, size_t n_line) { glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glTexParameterf(GLenum target, GLenum pname, GLfloat param, const char *p_s_file, size_t n_line) { glTexParameterf(target, pname, param); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glTexParameterfv(GLenum target, GLenum pname, const GLfloat* params, const char *p_s_file, size_t n_line) { glTexParameterfv(target, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glTexParameteri(GLenum target, GLenum pname, GLint param, const char *p_s_file, size_t n_line) { glTexParameteri(target, pname, param); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glTexParameteriv(GLenum target, GLenum pname, const GLint* params, const char *p_s_file, size_t n_line) { glTexParameteriv(target, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid* pixels, const char *p_s_file, size_t n_line) { glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glViewport(GLint x, GLint y, GLsizei width, GLsizei height, const char *p_s_file, size_t n_line) { glViewport(x, y, width, height); GL_CHECK_BLOCK(p_s_file, n_line);  }
// GL autocheck wrapper functions

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#define glBindTexture(target,texture) __glachwrap_glBindTexture(target, texture, __FILE__, __LINE__)
#define glBlendFunc(sfactor,dfactor) __glachwrap_glBlendFunc(sfactor, dfactor, __FILE__, __LINE__)
#define glClear(mask) __glachwrap_glClear(mask, __FILE__, __LINE__)
#define glClearColor(red,green,blue,alpha) __glachwrap_glClearColor(red, green, blue, alpha, __FILE__, __LINE__)
#define glClearStencil(s) __glachwrap_glClearStencil(s, __FILE__, __LINE__)
#define glColorMask(red,green,blue,alpha) __glachwrap_glColorMask(red, green, blue, alpha, __FILE__, __LINE__)
#define glCopyTexImage2D(target,level,internalformat,x,y,width,height,border) __glachwrap_glCopyTexImage2D(target, level, internalformat, x, y, width, height, border, __FILE__, __LINE__)
#define glCopyTexSubImage2D(target,level,xoffset,yoffset,x,y,width,height) __glachwrap_glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height, __FILE__, __LINE__)
#define glCullFace(mode) __glachwrap_glCullFace(mode, __FILE__, __LINE__)
#define glDeleteTextures(n,textures) __glachwrap_glDeleteTextures(n, textures, __FILE__, __LINE__)
#define glDepthFunc(func) __glachwrap_glDepthFunc(func, __FILE__, __LINE__)
#define glDepthMask(flag) __glachwrap_glDepthMask(flag, __FILE__, __LINE__)
#define glDisable(cap) __glachwrap_glDisable(cap, __FILE__, __LINE__)
#define glDrawArrays(mode,first,count) __glachwrap_glDrawArrays(mode, first, count, __FILE__, __LINE__)
#define glDrawElements(mode,count,type,indices) __glachwrap_glDrawElements(mode, count, type, indices, __FILE__, __LINE__)
#define glEnable(cap) __glachwrap_glEnable(cap, __FILE__, __LINE__)
#define glFinish() __glachwrap_glFinish(__FILE__, __LINE__)
#define glFlush() __glachwrap_glFlush(__FILE__, __LINE__)
#define glFrontFace(mode) __glachwrap_glFrontFace(mode, __FILE__, __LINE__)
#define glGenTextures(n,textures) __glachwrap_glGenTextures(n, textures, __FILE__, __LINE__)
#define glGetBooleanv(pname,params) __glachwrap_glGetBooleanv(pname, params, __FILE__, __LINE__)
//#define glGetError() __glachwrap_glGetError(__FILE__, __LINE__)
#define glGetFloatv(pname,params) __glachwrap_glGetFloatv(pname, params, __FILE__, __LINE__)
#define glGetIntegerv(pname,params) __glachwrap_glGetIntegerv(pname, params, __FILE__, __LINE__)
#define glGetString(name) __glachwrap_glGetString(name, __FILE__, __LINE__)
#define glGetTexParameterfv(target,pname,params) __glachwrap_glGetTexParameterfv(target, pname, params, __FILE__, __LINE__)
#define glGetTexParameteriv(target,pname,params) __glachwrap_glGetTexParameteriv(target, pname, params, __FILE__, __LINE__)
#define glHint(target,mode) __glachwrap_glHint(target, mode, __FILE__, __LINE__)
#define glIsEnabled(cap) __glachwrap_glIsEnabled(cap, __FILE__, __LINE__)
#define glIsTexture(texture) __glachwrap_glIsTexture(texture, __FILE__, __LINE__)
#define glLineWidth(width) __glachwrap_glLineWidth(width, __FILE__, __LINE__)
#define glPixelStorei(pname,param) __glachwrap_glPixelStorei(pname, param, __FILE__, __LINE__)
#define glPolygonOffset(factor,units) __glachwrap_glPolygonOffset(factor, units, __FILE__, __LINE__)
#define glReadPixels(x,y,width,height,format,type,pixels) __glachwrap_glReadPixels(x, y, width, height, format, type, pixels, __FILE__, __LINE__)
#define glScissor(x,y,width,height) __glachwrap_glScissor(x, y, width, height, __FILE__, __LINE__)
#define glStencilFunc(func,ref,mask) __glachwrap_glStencilFunc(func, ref, mask, __FILE__, __LINE__)
#define glStencilMask(mask) __glachwrap_glStencilMask(mask, __FILE__, __LINE__)
#define glStencilOp(fail,zfail,zpass) __glachwrap_glStencilOp(fail, zfail, zpass, __FILE__, __LINE__)
#define glTexImage2D(target,level,internalformat,width,height,border,format,type,pixels) __glachwrap_glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels, __FILE__, __LINE__)
#define glTexParameterf(target,pname,param) __glachwrap_glTexParameterf(target, pname, param, __FILE__, __LINE__)
#define glTexParameterfv(target,pname,params) __glachwrap_glTexParameterfv(target, pname, params, __FILE__, __LINE__)
#define glTexParameteri(target,pname,param) __glachwrap_glTexParameteri(target, pname, param, __FILE__, __LINE__)
#define glTexParameteriv(target,pname,params) __glachwrap_glTexParameteriv(target, pname, params, __FILE__, __LINE__)
#define glTexSubImage2D(target,level,xoffset,yoffset,width,height,format,type,pixels) __glachwrap_glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels, __FILE__, __LINE__)
#define glViewport(x,y,width,height) __glachwrap_glViewport(x, y, width, height, __FILE__, __LINE__)
// rename the originals to call the wrapper functions

#endif // GL_AUTOCHECK && (_DEBUG || FORCE_GL_CHECK)

#ifdef FIX_GL_FUNCTION_LINKING_CONFLICTS
#define glActiveTexture GLFuncPtrName(glActiveTexture)
#define glAttachShader GLFuncPtrName(glAttachShader)
#define glBindAttribLocation GLFuncPtrName(glBindAttribLocation)
#define glBindBuffer GLFuncPtrName(glBindBuffer)
#define glBindFramebuffer GLFuncPtrName(glBindFramebuffer)
#define glBindRenderbuffer GLFuncPtrName(glBindRenderbuffer)
#define glBlendColor GLFuncPtrName(glBlendColor)
#define glBlendEquation GLFuncPtrName(glBlendEquation)
#define glBlendEquationSeparate GLFuncPtrName(glBlendEquationSeparate)
#define glBlendFuncSeparate GLFuncPtrName(glBlendFuncSeparate)
#define glBufferData GLFuncPtrName(glBufferData)
#define glBufferSubData GLFuncPtrName(glBufferSubData)
#define glCheckFramebufferStatus GLFuncPtrName(glCheckFramebufferStatus)
#define glClearDepthf GLFuncPtrName(glClearDepthf)
#define glCompileShader GLFuncPtrName(glCompileShader)
#define glCompressedTexImage2D GLFuncPtrName(glCompressedTexImage2D)
#define glCompressedTexSubImage2D GLFuncPtrName(glCompressedTexSubImage2D)
#define glCreateProgram GLFuncPtrName(glCreateProgram)
#define glCreateShader GLFuncPtrName(glCreateShader)
#define glDeleteBuffers GLFuncPtrName(glDeleteBuffers)
#define glDeleteFramebuffers GLFuncPtrName(glDeleteFramebuffers)
#define glDeleteProgram GLFuncPtrName(glDeleteProgram)
#define glDeleteRenderbuffers GLFuncPtrName(glDeleteRenderbuffers)
#define glDeleteShader GLFuncPtrName(glDeleteShader)
#define glDepthRangef GLFuncPtrName(glDepthRangef)
#define glDetachShader GLFuncPtrName(glDetachShader)
#define glDisableVertexAttribArray GLFuncPtrName(glDisableVertexAttribArray)
#define glEnableVertexAttribArray GLFuncPtrName(glEnableVertexAttribArray)
#define glFramebufferRenderbuffer GLFuncPtrName(glFramebufferRenderbuffer)
#define glFramebufferTexture2D GLFuncPtrName(glFramebufferTexture2D)
#define glGenBuffers GLFuncPtrName(glGenBuffers)
#define glGenFramebuffers GLFuncPtrName(glGenFramebuffers)
#define glGenRenderbuffers GLFuncPtrName(glGenRenderbuffers)
#define glGenerateMipmap GLFuncPtrName(glGenerateMipmap)
#define glGetActiveAttrib GLFuncPtrName(glGetActiveAttrib)
#define glGetActiveUniform GLFuncPtrName(glGetActiveUniform)
#define glGetAttachedShaders GLFuncPtrName(glGetAttachedShaders)
#define glGetAttribLocation GLFuncPtrName(glGetAttribLocation)
#define glGetBufferParameteriv GLFuncPtrName(glGetBufferParameteriv)
#define glGetFramebufferAttachmentParameteriv GLFuncPtrName(glGetFramebufferAttachmentParameteriv)
#define glGetProgramInfoLog GLFuncPtrName(glGetProgramInfoLog)
#define glGetProgramiv GLFuncPtrName(glGetProgramiv)
#define glGetRenderbufferParameteriv GLFuncPtrName(glGetRenderbufferParameteriv)
#define glGetShaderInfoLog GLFuncPtrName(glGetShaderInfoLog)
#define glGetShaderPrecisionFormat GLFuncPtrName(glGetShaderPrecisionFormat)
#define glGetShaderSource GLFuncPtrName(glGetShaderSource)
#define glGetShaderiv GLFuncPtrName(glGetShaderiv)
#define glGetUniformLocation GLFuncPtrName(glGetUniformLocation)
#define glGetUniformfv GLFuncPtrName(glGetUniformfv)
#define glGetUniformiv GLFuncPtrName(glGetUniformiv)
#define glGetVertexAttribPointerv GLFuncPtrName(glGetVertexAttribPointerv)
#define glGetVertexAttribfv GLFuncPtrName(glGetVertexAttribfv)
#define glGetVertexAttribiv GLFuncPtrName(glGetVertexAttribiv)
#define glIsBuffer GLFuncPtrName(glIsBuffer)
#define glIsFramebuffer GLFuncPtrName(glIsFramebuffer)
#define glIsProgram GLFuncPtrName(glIsProgram)
#define glIsRenderbuffer GLFuncPtrName(glIsRenderbuffer)
#define glIsShader GLFuncPtrName(glIsShader)
#define glLinkProgram GLFuncPtrName(glLinkProgram)
#define glReleaseShaderCompiler GLFuncPtrName(glReleaseShaderCompiler)
#define glRenderbufferStorage GLFuncPtrName(glRenderbufferStorage)
#define glSampleCoverage GLFuncPtrName(glSampleCoverage)
#define glShaderBinary GLFuncPtrName(glShaderBinary)
#define glShaderSource GLFuncPtrName(glShaderSource)
#define glStencilFuncSeparate GLFuncPtrName(glStencilFuncSeparate)
#define glStencilMaskSeparate GLFuncPtrName(glStencilMaskSeparate)
#define glStencilOpSeparate GLFuncPtrName(glStencilOpSeparate)
#define glUniform1f GLFuncPtrName(glUniform1f)
#define glUniform1fv GLFuncPtrName(glUniform1fv)
#define glUniform1i GLFuncPtrName(glUniform1i)
#define glUniform1iv GLFuncPtrName(glUniform1iv)
#define glUniform2f GLFuncPtrName(glUniform2f)
#define glUniform2fv GLFuncPtrName(glUniform2fv)
#define glUniform2i GLFuncPtrName(glUniform2i)
#define glUniform2iv GLFuncPtrName(glUniform2iv)
#define glUniform3f GLFuncPtrName(glUniform3f)
#define glUniform3fv GLFuncPtrName(glUniform3fv)
#define glUniform3i GLFuncPtrName(glUniform3i)
#define glUniform3iv GLFuncPtrName(glUniform3iv)
#define glUniform4f GLFuncPtrName(glUniform4f)
#define glUniform4fv GLFuncPtrName(glUniform4fv)
#define glUniform4i GLFuncPtrName(glUniform4i)
#define glUniform4iv GLFuncPtrName(glUniform4iv)
#define glUniformMatrix2fv GLFuncPtrName(glUniformMatrix2fv)
#define glUniformMatrix3fv GLFuncPtrName(glUniformMatrix3fv)
#define glUniformMatrix4fv GLFuncPtrName(glUniformMatrix4fv)
#define glUseProgram GLFuncPtrName(glUseProgram)
#define glValidateProgram GLFuncPtrName(glValidateProgram)
#define glVertexAttrib1f GLFuncPtrName(glVertexAttrib1f)
#define glVertexAttrib1fv GLFuncPtrName(glVertexAttrib1fv)
#define glVertexAttrib2f GLFuncPtrName(glVertexAttrib2f)
#define glVertexAttrib2fv GLFuncPtrName(glVertexAttrib2fv)
#define glVertexAttrib3f GLFuncPtrName(glVertexAttrib3f)
#define glVertexAttrib3fv GLFuncPtrName(glVertexAttrib3fv)
#define glVertexAttrib4f GLFuncPtrName(glVertexAttrib4f)
#define glVertexAttrib4fv GLFuncPtrName(glVertexAttrib4fv)
#define glVertexAttribPointer GLFuncPtrName(glVertexAttribPointer)
#endif //FIX_GL_FUNCTION_LINKING_CONFLICTS

#if defined(GL_AUTOCHECK) && (defined(_DEBUG) || defined(FORCE_GL_CHECK)) && !defined(FORCE_AUTOCHECK_OFF_ON_GL_EXTENSIONS)

#ifdef __cplusplus
}
#endif // __cplusplus

inline void __glachwrap_glActiveTexture(GLenum texunit, const char *p_s_file, size_t n_line) { glActiveTexture(texunit); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glAttachShader(GLuint program, GLuint shader, const char *p_s_file, size_t n_line) { glAttachShader(program, shader); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBindAttribLocation(GLuint program, GLuint index, const GLchar* name, const char *p_s_file, size_t n_line) { glBindAttribLocation(program, index, name); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBindBuffer(GLenum target, GLuint name, const char *p_s_file, size_t n_line) { glBindBuffer(target, name); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBindFramebuffer(GLenum target, GLuint name, const char *p_s_file, size_t n_line) { glBindFramebuffer(target, name); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBindRenderbuffer(GLenum target, GLuint name, const char *p_s_file, size_t n_line) { glBindRenderbuffer(target, name); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBlendColor(GLclampf r, GLclampf g, GLclampf b, GLclampf a, const char *p_s_file, size_t n_line) { glBlendColor(r, g, b, a); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBlendEquation(GLenum name, const char *p_s_file, size_t n_line) { glBlendEquation(name); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBlendEquationSeparate(GLenum modeRGB, GLenum modeAlpha, const char *p_s_file, size_t n_line) { glBlendEquationSeparate(modeRGB, modeAlpha); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBlendFuncSeparate(GLenum nameR, GLenum nameG, GLenum nameB, GLenum nameA, const char *p_s_file, size_t n_line) { glBlendFuncSeparate(nameR, nameG, nameB, nameA); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBufferData(GLenum target, GLsizeiptr size, const GLvoid*  data, GLenum usage, const char *p_s_file, size_t n_line) { glBufferData(target, size, data, usage); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, const GLvoid*  data, const char *p_s_file, size_t n_line) { glBufferSubData(target, offset, size, data); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline GLenum __glachwrap_glCheckFramebufferStatus(GLenum target, const char *p_s_file, size_t n_line) { GLenum t_result = glCheckFramebufferStatus(target); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glClearDepthf(GLclampf ref, const char *p_s_file, size_t n_line) { glClearDepthf(ref); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glCompileShader(GLuint shader, const char *p_s_file, size_t n_line) { glCompileShader(shader); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glCompressedTexImage2D(GLenum target, GLint level, GLenum internalfmt, GLsizei width, GLsizei height, GLint border, GLsizei imagesize, const GLvoid*  data, const char *p_s_file, size_t n_line) { glCompressedTexImage2D(target, level, internalfmt, width, height, border, imagesize, data); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glCompressedTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imageSize, const GLvoid*  data, const char *p_s_file, size_t n_line) { glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline GLuint __glachwrap_glCreateProgram(const char *p_s_file, size_t n_line) { GLuint t_result = glCreateProgram(); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline GLuint __glachwrap_glCreateShader(GLenum type, const char *p_s_file, size_t n_line) { GLuint t_result = glCreateShader(type); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glDeleteBuffers(GLsizei count, const GLuint*  names, const char *p_s_file, size_t n_line) {/* glDeleteBuffers(count, names); GL_CHECK_BLOCK(p_s_file, n_line);  */}
inline void __glachwrap_glDeleteFramebuffers(GLsizei count, const GLuint*  names, const char *p_s_file, size_t n_line) { glDeleteFramebuffers(count, names); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDeleteProgram(GLuint program, const char *p_s_file, size_t n_line) { /*glDeleteProgram(program); GL_CHECK_BLOCK(p_s_file, n_line); */ }
inline void __glachwrap_glDeleteRenderbuffers(GLsizei count, const GLuint*  names, const char *p_s_file, size_t n_line) { glDeleteRenderbuffers(count, names); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDeleteShader(GLuint shader, const char *p_s_file, size_t n_line) { /*glDeleteShader(shader); GL_CHECK_BLOCK(p_s_file, n_line); */ }
//inline void __glachwrap_glDepthRangef(GLclampf near, GLclampf far, const char *p_s_file, size_t n_line) { glDepthRangef(near, far); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDetachShader(GLuint program, GLuint shader, const char *p_s_file, size_t n_line) { glDetachShader(program, shader); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDisableVertexAttribArray(GLuint index, const char *p_s_file, size_t n_line) { glDisableVertexAttribArray(index); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glEnableVertexAttribArray(GLuint index, const char *p_s_file, size_t n_line) { glEnableVertexAttribArray(index); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glFramebufferRenderbuffer(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer, const char *p_s_file, size_t n_line) { glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glFramebufferTexture2D(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, const char *p_s_file, size_t n_line) { glFramebufferTexture2D(target, attachment, textarget, texture, level); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGenBuffers(GLsizei count, GLuint*  names, const char *p_s_file, size_t n_line) { glGenBuffers(count, names); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGenerateMipmap(GLenum target, const char *p_s_file, size_t n_line) { glGenerateMipmap(target); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGenFramebuffers(GLsizei count, GLuint*  names, const char *p_s_file, size_t n_line) { glGenFramebuffers(count, names); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGenRenderbuffers(GLsizei count, GLuint*  names, const char *p_s_file, size_t n_line) { glGenRenderbuffers(count, names); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetActiveAttrib(GLuint program, GLuint index, GLsizei bufSize, GLsizei* length, GLsizei* size, GLenum* type, GLchar* name, const char *p_s_file, size_t n_line) { glGetActiveAttrib(program, index, bufSize, length, size, type, name); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetActiveUniform(GLuint program, GLuint index, GLsizei bufSize, GLsizei* length, GLsizei* size, GLenum* type, GLchar* name, const char *p_s_file, size_t n_line) { glGetActiveUniform(program, index, bufSize, length, size, type, name); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetAttachedShaders(GLuint program, GLsizei maxCount, GLsizei* count, GLuint* shaders, const char *p_s_file, size_t n_line) { glGetAttachedShaders(program, maxCount, count, shaders); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline GLint __glachwrap_glGetAttribLocation(GLuint program, const GLchar* name, const char *p_s_file, size_t n_line) { GLint t_result = glGetAttribLocation(program, name); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glGetBufferParameteriv(GLenum target, GLenum name, GLint*  value, const char *p_s_file, size_t n_line) { glGetBufferParameteriv(target, name, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetFramebufferAttachmentParameteriv(GLenum target, GLenum attachment, GLenum name, GLint* value, const char *p_s_file, size_t n_line) { glGetFramebufferAttachmentParameteriv(target, attachment, name, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetProgramiv(GLuint program, GLenum pname, GLint* params, const char *p_s_file, size_t n_line) { glGetProgramiv(program, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetProgramInfoLog(GLuint program, GLsizei bufSize, GLsizei* length, GLchar* infoLog, const char *p_s_file, size_t n_line) { glGetProgramInfoLog(program, bufSize, length, infoLog); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetRenderbufferParameteriv(GLenum target, GLenum name, GLint* value, const char *p_s_file, size_t n_line) { glGetRenderbufferParameteriv(target, name, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetShaderiv(GLuint program, GLenum pname, GLint* params, const char *p_s_file, size_t n_line) { glGetShaderiv(program, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetShaderInfoLog(GLuint shader, GLsizei bufSize, GLsizei* length, GLchar* infoLog, const char *p_s_file, size_t n_line) { glGetShaderInfoLog(shader, bufSize, length, infoLog); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetShaderPrecisionFormat(GLenum shaderType, GLenum precisionType, int*  range, int*  precision, const char *p_s_file, size_t n_line) { glGetShaderPrecisionFormat(shaderType, precisionType, range, precision); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetShaderSource(GLuint shader, GLsizei bufSize, GLsizei* length, GLchar* source, const char *p_s_file, size_t n_line) { glGetShaderSource(shader, bufSize, length, source); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetUniformfv(GLuint program, GLint location, GLfloat* params, const char *p_s_file, size_t n_line) { glGetUniformfv(program, location, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetUniformiv(GLuint program, GLint location, GLint* params, const char *p_s_file, size_t n_line) { glGetUniformiv(program, location, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline GLint __glachwrap_glGetUniformLocation(GLuint program, const GLchar* name, const char *p_s_file, size_t n_line) { GLint t_result = glGetUniformLocation(program, name); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glGetVertexAttribfv(GLuint index, GLenum pname, GLfloat* params, const char *p_s_file, size_t n_line) { glGetVertexAttribfv(index, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetVertexAttribiv(GLuint index, GLenum pname, GLint* params, const char *p_s_file, size_t n_line) { glGetVertexAttribiv(index, pname, params); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGetVertexAttribPointerv(GLuint index, GLenum pname, GLvoid**  pointer, const char *p_s_file, size_t n_line) { glGetVertexAttribPointerv(index, pname, pointer); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline GLboolean __glachwrap_glIsBuffer(GLuint name, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsBuffer(name); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline GLboolean __glachwrap_glIsFramebuffer(GLuint name, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsFramebuffer(name); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline GLboolean __glachwrap_glIsProgram(GLuint program, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsProgram(program); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline GLboolean __glachwrap_glIsRenderbuffer(GLuint name, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsRenderbuffer(name); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline GLboolean __glachwrap_glIsShader(GLuint shader, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsShader(shader); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
inline void __glachwrap_glLinkProgram(GLuint program, const char *p_s_file, size_t n_line) { glLinkProgram(program); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glReleaseShaderCompiler(const char *p_s_file, size_t n_line) { glReleaseShaderCompiler(); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glRenderbufferStorage(GLenum target, GLenum internalformat, GLsizei width, GLsizei height, const char *p_s_file, size_t n_line) { glRenderbufferStorage(target, internalformat, width, height); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glSampleCoverage(GLclampf value, GLboolean invert, const char *p_s_file, size_t n_line) { glSampleCoverage(value, invert); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glShaderBinary(GLsizei n, const GLuint*  shaders, GLenum binatyformat, const void*  binary, GLsizei length, const char *p_s_file, size_t n_line) { glShaderBinary(n, shaders, binatyformat, binary, length); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glShaderSource(GLuint shader, GLsizei count, const GLchar** string, const GLint* length, const char *p_s_file, size_t n_line) { glShaderSource(shader, count, string, length); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glStencilFuncSeparate(GLenum face, GLenum func, GLint ref, GLuint mask, const char *p_s_file, size_t n_line) { glStencilFuncSeparate(face, func, ref, mask); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glStencilMaskSeparate(GLenum face, GLuint mask, const char *p_s_file, size_t n_line) { glStencilMaskSeparate(face, mask); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glStencilOpSeparate(GLenum face, GLenum fail, GLenum zfail, GLenum zpass, const char *p_s_file, size_t n_line) { glStencilOpSeparate(face, fail, zfail, zpass); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform1f(GLint location, GLfloat v0, const char *p_s_file, size_t n_line) { glUniform1f(location, v0); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform2f(GLint location, GLfloat v0, GLfloat v1, const char *p_s_file, size_t n_line) { glUniform2f(location, v0, v1); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform3f(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, const char *p_s_file, size_t n_line) { glUniform3f(location, v0, v1, v2); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform4f(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3, const char *p_s_file, size_t n_line) { glUniform4f(location, v0, v1, v2, v3); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform1i(GLint location, GLint v0, const char *p_s_file, size_t n_line) { glUniform1i(location, v0); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform2i(GLint location, GLint v0, GLint v1, const char *p_s_file, size_t n_line) { glUniform2i(location, v0, v1); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform3i(GLint location, GLint v0, GLint v1, GLint v2, const char *p_s_file, size_t n_line) { glUniform3i(location, v0, v1, v2); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform4i(GLint location, GLint v0, GLint v1, GLint v2, GLint v3, const char *p_s_file, size_t n_line) { glUniform4i(location, v0, v1, v2, v3); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform1fv(GLint location, GLsizei count, const GLfloat* value, const char *p_s_file, size_t n_line) { glUniform1fv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform2fv(GLint location, GLsizei count, const GLfloat* value, const char *p_s_file, size_t n_line) { glUniform2fv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform3fv(GLint location, GLsizei count, const GLfloat* value, const char *p_s_file, size_t n_line) { glUniform3fv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform4fv(GLint location, GLsizei count, const GLfloat* value, const char *p_s_file, size_t n_line) { glUniform4fv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform1iv(GLint location, GLsizei count, const GLint* value, const char *p_s_file, size_t n_line) { glUniform1iv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform2iv(GLint location, GLsizei count, const GLint* value, const char *p_s_file, size_t n_line) { glUniform2iv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform3iv(GLint location, GLsizei count, const GLint* value, const char *p_s_file, size_t n_line) { glUniform3iv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniform4iv(GLint location, GLsizei count, const GLint* value, const char *p_s_file, size_t n_line) { glUniform4iv(location, count, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat* value, const char *p_s_file, size_t n_line) { glUniformMatrix2fv(location, count, transpose, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat* value, const char *p_s_file, size_t n_line) { glUniformMatrix3fv(location, count, transpose, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat* value, const char *p_s_file, size_t n_line) { glUniformMatrix4fv(location, count, transpose, value); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glUseProgram(GLuint program, const char *p_s_file, size_t n_line) { glUseProgram(program); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glValidateProgram(GLuint program, const char *p_s_file, size_t n_line) { glValidateProgram(program); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib1f(GLuint index, GLfloat x, const char *p_s_file, size_t n_line) { glVertexAttrib1f(index, x); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib1fv(GLuint index, const GLfloat* v, const char *p_s_file, size_t n_line) { glVertexAttrib1fv(index, v); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib2f(GLuint index, GLfloat x, GLfloat y, const char *p_s_file, size_t n_line) { glVertexAttrib2f(index, x, y); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib2fv(GLuint index, const GLfloat* v, const char *p_s_file, size_t n_line) { glVertexAttrib2fv(index, v); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib3f(GLuint index, GLfloat x, GLfloat y, GLfloat z, const char *p_s_file, size_t n_line) { glVertexAttrib3f(index, x, y, z); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib3fv(GLuint index, const GLfloat* v, const char *p_s_file, size_t n_line) { glVertexAttrib3fv(index, v); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib4f(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w, const char *p_s_file, size_t n_line) { glVertexAttrib4f(index, x, y, z, w); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttrib4fv(GLuint index, const GLfloat* v, const char *p_s_file, size_t n_line) { glVertexAttrib4fv(index, v); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glVertexAttribPointer(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, const GLvoid* pointer, const char *p_s_file, size_t n_line) { glVertexAttribPointer(index, size, type, normalized, stride, pointer); GL_CHECK_BLOCK(p_s_file, n_line);  }
// GL autocheck wrapper functions

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#ifdef FIX_GL_FUNCTION_LINKING_CONFLICTS
#undef glActiveTexture
#undef glAttachShader
#undef glBindAttribLocation
#undef glBindBuffer
#undef glBindFramebuffer
#undef glBindRenderbuffer
#undef glBlendColor
#undef glBlendEquation
#undef glBlendEquationSeparate
#undef glBlendFuncSeparate
#undef glBufferData
#undef glBufferSubData
#undef glCheckFramebufferStatus
#undef glClearDepthf
#undef glCompileShader
#undef glCompressedTexImage2D
#undef glCompressedTexSubImage2D
#undef glCreateProgram
#undef glCreateShader
#undef glDeleteBuffers
#undef glDeleteFramebuffers
#undef glDeleteProgram
#undef glDeleteRenderbuffers
#undef glDeleteShader
#undef glDepthRangef
#undef glDetachShader
#undef glDisableVertexAttribArray
#undef glEnableVertexAttribArray
#undef glFramebufferRenderbuffer
#undef glFramebufferTexture2D
#undef glGenBuffers
#undef glGenerateMipmap
#undef glGenFramebuffers
#undef glGenRenderbuffers
#undef glGetActiveAttrib
#undef glGetActiveUniform
#undef glGetAttachedShaders
#undef glGetAttribLocation
#undef glGetBufferParameteriv
#undef glGetFramebufferAttachmentParameteriv
#undef glGetProgramiv
#undef glGetProgramInfoLog
#undef glGetRenderbufferParameteriv
#undef glGetShaderiv
#undef glGetShaderInfoLog
#undef glGetShaderPrecisionFormat
#undef glGetShaderSource
#undef glGetUniformfv
#undef glGetUniformiv
#undef glGetUniformLocation
#undef glGetVertexAttribfv
#undef glGetVertexAttribiv
#undef glGetVertexAttribPointerv
#undef glIsBuffer
#undef glIsFramebuffer
#undef glIsProgram
#undef glIsRenderbuffer
#undef glIsShader
#undef glLinkProgram
#undef glReleaseShaderCompiler
#undef glRenderbufferStorage
#undef glSampleCoverage
#undef glShaderBinary
#undef glShaderSource
#undef glStencilFuncSeparate
#undef glStencilMaskSeparate
#undef glStencilOpSeparate
#undef glUniform1f
#undef glUniform2f
#undef glUniform3f
#undef glUniform4f
#undef glUniform1i
#undef glUniform2i
#undef glUniform3i
#undef glUniform4i
#undef glUniform1fv
#undef glUniform2fv
#undef glUniform3fv
#undef glUniform4fv
#undef glUniform1iv
#undef glUniform2iv
#undef glUniform3iv
#undef glUniform4iv
#undef glUniformMatrix2fv
#undef glUniformMatrix3fv
#undef glUniformMatrix4fv
#undef glUseProgram
#undef glValidateProgram
#undef glVertexAttrib1f
#undef glVertexAttrib1fv
#undef glVertexAttrib2f
#undef glVertexAttrib2fv
#undef glVertexAttrib3f
#undef glVertexAttrib3fv
#undef glVertexAttrib4f
#undef glVertexAttrib4fv
#undef glVertexAttribPointer
#endif // FIX_GL_FUNCTION_LINKING_CONFLICTS
// undef function names as they need to be renamed again

#define glActiveTexture(texunit) __glachwrap_glActiveTexture(texunit, __FILE__, __LINE__)
#define glAttachShader(program,shader) __glachwrap_glAttachShader(program, shader, __FILE__, __LINE__)
#define glBindAttribLocation(program,index,name) __glachwrap_glBindAttribLocation(program, index, name, __FILE__, __LINE__)
#define glBindBuffer(target,name) __glachwrap_glBindBuffer(target, name, __FILE__, __LINE__)
#define glBindFramebuffer(target,name) __glachwrap_glBindFramebuffer(target, name, __FILE__, __LINE__)
#define glBindRenderbuffer(target,name) __glachwrap_glBindRenderbuffer(target, name, __FILE__, __LINE__)
#define glBlendColor(r,g,b,a) __glachwrap_glBlendColor(r, g, b, a, __FILE__, __LINE__)
#define glBlendEquation(name) __glachwrap_glBlendEquation(name, __FILE__, __LINE__)
#define glBlendEquationSeparate(modeRGB,modeAlpha) __glachwrap_glBlendEquationSeparate(modeRGB, modeAlpha, __FILE__, __LINE__)
#define glBlendFuncSeparate(nameR,nameG,nameB,nameA) __glachwrap_glBlendFuncSeparate(nameR, nameG, nameB, nameA, __FILE__, __LINE__)
#define glBufferData(target,size,data,usage) __glachwrap_glBufferData(target, size, data, usage, __FILE__, __LINE__)
#define glBufferSubData(target,offset,size,data) __glachwrap_glBufferSubData(target, offset, size, data, __FILE__, __LINE__)
#define glCheckFramebufferStatus(target) __glachwrap_glCheckFramebufferStatus(target, __FILE__, __LINE__)
#define glClearDepthf(ref) __glachwrap_glClearDepthf(ref, __FILE__, __LINE__)
#define glCompileShader(shader) __glachwrap_glCompileShader(shader, __FILE__, __LINE__)
#define glCompressedTexImage2D(target,level,internalfmt,width,height,border,imagesize,data) __glachwrap_glCompressedTexImage2D(target, level, internalfmt, width, height, border, imagesize, data, __FILE__, __LINE__)
#define glCompressedTexSubImage2D(target,level,xoffset,yoffset,width,height,format,imageSize,data) __glachwrap_glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data, __FILE__, __LINE__)
#define glCreateProgram() __glachwrap_glCreateProgram(__FILE__, __LINE__)
#define glCreateShader(type) __glachwrap_glCreateShader(type, __FILE__, __LINE__)
#define glDeleteBuffers(count,names) __glachwrap_glDeleteBuffers(count, names, __FILE__, __LINE__)
#define glDeleteFramebuffers(count,names) __glachwrap_glDeleteFramebuffers(count, names, __FILE__, __LINE__)
#define glDeleteProgram(program) __glachwrap_glDeleteProgram(program, __FILE__, __LINE__)
#define glDeleteRenderbuffers(count,names) __glachwrap_glDeleteRenderbuffers(count, names, __FILE__, __LINE__)
#define glDeleteShader(shader) __glachwrap_glDeleteShader(shader, __FILE__, __LINE__)
#define glDepthRangef(near,far) __glachwrap_glDepthRangef(near, far, __FILE__, __LINE__)
#define glDetachShader(program,shader) __glachwrap_glDetachShader(program, shader, __FILE__, __LINE__)
#define glDisableVertexAttribArray(index) __glachwrap_glDisableVertexAttribArray(index, __FILE__, __LINE__)
#define glEnableVertexAttribArray(index) __glachwrap_glEnableVertexAttribArray(index, __FILE__, __LINE__)
#define glFramebufferRenderbuffer(target,attachment,renderbuffertarget,renderbuffer) __glachwrap_glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer, __FILE__, __LINE__)
#define glFramebufferTexture2D(target,attachment,textarget,texture,level) __glachwrap_glFramebufferTexture2D(target, attachment, textarget, texture, level, __FILE__, __LINE__)
#define glGenBuffers(count,names) __glachwrap_glGenBuffers(count, names, __FILE__, __LINE__)
#define glGenerateMipmap(target) __glachwrap_glGenerateMipmap(target, __FILE__, __LINE__)
#define glGenFramebuffers(count,names) __glachwrap_glGenFramebuffers(count, names, __FILE__, __LINE__)
#define glGenRenderbuffers(count,names) __glachwrap_glGenRenderbuffers(count, names, __FILE__, __LINE__)
#define glGetActiveAttrib(program,index,bufSize,length,size,type,name) __glachwrap_glGetActiveAttrib(program, index, bufSize, length, size, type, name, __FILE__, __LINE__)
#define glGetActiveUniform(program,index,bufSize,length,size,type,name) __glachwrap_glGetActiveUniform(program, index, bufSize, length, size, type, name, __FILE__, __LINE__)
#define glGetAttachedShaders(program,maxCount,count,shaders) __glachwrap_glGetAttachedShaders(program, maxCount, count, shaders, __FILE__, __LINE__)
#define glGetAttribLocation(program,name) __glachwrap_glGetAttribLocation(program, name, __FILE__, __LINE__)
#define glGetBufferParameteriv(target,name,value) __glachwrap_glGetBufferParameteriv(target, name, value, __FILE__, __LINE__)
#define glGetFramebufferAttachmentParameteriv(target,attachment,name,value) __glachwrap_glGetFramebufferAttachmentParameteriv(target, attachment, name, value, __FILE__, __LINE__)
#define glGetProgramiv(program,pname,params) __glachwrap_glGetProgramiv(program, pname, params, __FILE__, __LINE__)
#define glGetProgramInfoLog(program,bufSize,length,infoLog) __glachwrap_glGetProgramInfoLog(program, bufSize, length, infoLog, __FILE__, __LINE__)
#define glGetRenderbufferParameteriv(target,name,value) __glachwrap_glGetRenderbufferParameteriv(target, name, value, __FILE__, __LINE__)
#define glGetShaderiv(program,pname,params) __glachwrap_glGetShaderiv(program, pname, params, __FILE__, __LINE__)
#define glGetShaderInfoLog(shader,bufSize,length,infoLog) __glachwrap_glGetShaderInfoLog(shader, bufSize, length, infoLog, __FILE__, __LINE__)
#define glGetShaderPrecisionFormat(shaderType,precisionType,range,precision) __glachwrap_glGetShaderPrecisionFormat(shaderType, precisionType, range, precision, __FILE__, __LINE__)
#define glGetShaderSource(shader,bufSize,length,source) __glachwrap_glGetShaderSource(shader, bufSize, length, source, __FILE__, __LINE__)
#define glGetUniformfv(program,location,params) __glachwrap_glGetUniformfv(program, location, params, __FILE__, __LINE__)
#define glGetUniformiv(program,location,params) __glachwrap_glGetUniformiv(program, location, params, __FILE__, __LINE__)
#define glGetUniformLocation(program,name) __glachwrap_glGetUniformLocation(program, name, __FILE__, __LINE__)
#define glGetVertexAttribfv(index,pname,params) __glachwrap_glGetVertexAttribfv(index, pname, params, __FILE__, __LINE__)
#define glGetVertexAttribiv(index,pname,params) __glachwrap_glGetVertexAttribiv(index, pname, params, __FILE__, __LINE__)
#define glGetVertexAttribPointerv(index,pname,pointer) __glachwrap_glGetVertexAttribPointerv(index, pname, pointer, __FILE__, __LINE__)
#define glIsBuffer(name) __glachwrap_glIsBuffer(name, __FILE__, __LINE__)
#define glIsFramebuffer(name) __glachwrap_glIsFramebuffer(name, __FILE__, __LINE__)
#define glIsProgram(program) __glachwrap_glIsProgram(program, __FILE__, __LINE__)
#define glIsRenderbuffer(name) __glachwrap_glIsRenderbuffer(name, __FILE__, __LINE__)
#define glIsShader(shader) __glachwrap_glIsShader(shader, __FILE__, __LINE__)
#define glLinkProgram(program) __glachwrap_glLinkProgram(program, __FILE__, __LINE__)
#define glReleaseShaderCompiler() __glachwrap_glReleaseShaderCompiler(__FILE__, __LINE__)
#define glRenderbufferStorage(target,internalformat,width,height) __glachwrap_glRenderbufferStorage(target, internalformat, width, height, __FILE__, __LINE__)
#define glSampleCoverage(value,invert) __glachwrap_glSampleCoverage(value, invert, __FILE__, __LINE__)
#define glShaderBinary(n,shaders,binatyformat,binary,length) __glachwrap_glShaderBinary(n, shaders, binatyformat, binary, length, __FILE__, __LINE__)
#define glShaderSource(shader,count,string,length) __glachwrap_glShaderSource(shader, count, string, length, __FILE__, __LINE__)
#define glStencilFuncSeparate(face,func,ref,mask) __glachwrap_glStencilFuncSeparate(face, func, ref, mask, __FILE__, __LINE__)
#define glStencilMaskSeparate(face,mask) __glachwrap_glStencilMaskSeparate(face, mask, __FILE__, __LINE__)
#define glStencilOpSeparate(face,fail,zfail,zpass) __glachwrap_glStencilOpSeparate(face, fail, zfail, zpass, __FILE__, __LINE__)
#define glUniform1f(location,v0) __glachwrap_glUniform1f(location, v0, __FILE__, __LINE__)
#define glUniform2f(location,v0,v1) __glachwrap_glUniform2f(location, v0, v1, __FILE__, __LINE__)
#define glUniform3f(location,v0,v1,v2) __glachwrap_glUniform3f(location, v0, v1, v2, __FILE__, __LINE__)
#define glUniform4f(location,v0,v1,v2,v3) __glachwrap_glUniform4f(location, v0, v1, v2, v3, __FILE__, __LINE__)
#define glUniform1i(location,v0) __glachwrap_glUniform1i(location, v0, __FILE__, __LINE__)
#define glUniform2i(location,v0,v1) __glachwrap_glUniform2i(location, v0, v1, __FILE__, __LINE__)
#define glUniform3i(location,v0,v1,v2) __glachwrap_glUniform3i(location, v0, v1, v2, __FILE__, __LINE__)
#define glUniform4i(location,v0,v1,v2,v3) __glachwrap_glUniform4i(location, v0, v1, v2, v3, __FILE__, __LINE__)
#define glUniform1fv(location,count,value) __glachwrap_glUniform1fv(location, count, value, __FILE__, __LINE__)
#define glUniform2fv(location,count,value) __glachwrap_glUniform2fv(location, count, value, __FILE__, __LINE__)
#define glUniform3fv(location,count,value) __glachwrap_glUniform3fv(location, count, value, __FILE__, __LINE__)
#define glUniform4fv(location,count,value) __glachwrap_glUniform4fv(location, count, value, __FILE__, __LINE__)
#define glUniform1iv(location,count,value) __glachwrap_glUniform1iv(location, count, value, __FILE__, __LINE__)
#define glUniform2iv(location,count,value) __glachwrap_glUniform2iv(location, count, value, __FILE__, __LINE__)
#define glUniform3iv(location,count,value) __glachwrap_glUniform3iv(location, count, value, __FILE__, __LINE__)
#define glUniform4iv(location,count,value) __glachwrap_glUniform4iv(location, count, value, __FILE__, __LINE__)
#define glUniformMatrix2fv(location,count,transpose,value) __glachwrap_glUniformMatrix2fv(location, count, transpose, value, __FILE__, __LINE__)
#define glUniformMatrix3fv(location,count,transpose,value) __glachwrap_glUniformMatrix3fv(location, count, transpose, value, __FILE__, __LINE__)
#define glUniformMatrix4fv(location,count,transpose,value) __glachwrap_glUniformMatrix4fv(location, count, transpose, value, __FILE__, __LINE__)
#define glUseProgram(program) __glachwrap_glUseProgram(program, __FILE__, __LINE__)
#define glValidateProgram(program) __glachwrap_glValidateProgram(program, __FILE__, __LINE__)
#define glVertexAttrib1f(index,x) __glachwrap_glVertexAttrib1f(index, x, __FILE__, __LINE__)
#define glVertexAttrib1fv(index,v) __glachwrap_glVertexAttrib1fv(index, v, __FILE__, __LINE__)
#define glVertexAttrib2f(index,x,y) __glachwrap_glVertexAttrib2f(index, x, y, __FILE__, __LINE__)
#define glVertexAttrib2fv(index,v) __glachwrap_glVertexAttrib2fv(index, v, __FILE__, __LINE__)
#define glVertexAttrib3f(index,x,y,z) __glachwrap_glVertexAttrib3f(index, x, y, z, __FILE__, __LINE__)
#define glVertexAttrib3fv(index,v) __glachwrap_glVertexAttrib3fv(index, v, __FILE__, __LINE__)
#define glVertexAttrib4f(index,x,y,z,w) __glachwrap_glVertexAttrib4f(index, x, y, z, w, __FILE__, __LINE__)
#define glVertexAttrib4fv(index,v) __glachwrap_glVertexAttrib4fv(index, v, __FILE__, __LINE__)
#define glVertexAttribPointer(index,size,type,normalized,stride,pointer) __glachwrap_glVertexAttribPointer(index, size, type, normalized, stride, pointer, __FILE__, __LINE__)
// rename the originals to call the wrapper functions

#endif // GL_AUTOCHECK && (_DEBUG || FORCE_GL_CHECK) && !FORCE_AUTOCHECK_OFF_ON_GL_EXTENSIONS

#ifdef __cplusplus
}
#endif

#endif // GL_ES_VERSION_2_0

//								--- GL_ARB_vertex_array_object (part of OpenGL 3.0 core) ---

#ifndef GL_ARB_vertex_array_object
#define GL_ARB_vertex_array_object 1
#define __GENERATE_GL_ARB_vertex_array_object__

extern bool GLEH_ARB_VERTEX_ARRAY_OBJECT;

#define GL_VERTEX_ARRAY_BINDING           0x85B5

extern void (GLApi *GLFuncPtrName(glBindVertexArray))(GLuint);
extern void (GLApi *GLFuncPtrName(glDeleteVertexArrays))(GLsizei, const GLuint *);
extern void (GLApi *GLFuncPtrName(glGenVertexArrays))(GLsizei, GLuint *);
extern GLboolean (GLApi *GLFuncPtrName(glIsVertexArray))(GLuint);

extern GLuint __n_gles20emu_default_vertex_array;

#ifdef FIX_GL_FUNCTION_LINKING_CONFLICTS
#define glBindVertexArray GLFuncPtrName(glBindVertexArray)
#define glDeleteVertexArrays GLFuncPtrName(glDeleteVertexArrays)
#define glGenVertexArrays GLFuncPtrName(glGenVertexArrays)
#define glIsVertexArray GLFuncPtrName(glIsVertexArray)
#endif // FIX_GL_FUNCTION_LINKING_CONFLICTS

#if defined(GL_AUTOCHECK) && (defined(_DEBUG) || defined(FORCE_GL_CHECK)) && !defined(FORCE_AUTOCHECK_OFF_ON_GL_EXTENSIONS)

inline void __glachwrap_glBindVertexArray(GLuint array, const char *p_s_file, size_t n_line) { if(array) glBindVertexArray(array); else glBindVertexArray(__n_gles20emu_default_vertex_array); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glDeleteVertexArrays(GLsizei count, const GLuint* arrays, const char *p_s_file, size_t n_line) { glDeleteVertexArrays(count, arrays); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline void __glachwrap_glGenVertexArrays(GLsizei count, GLuint* arrays, const char *p_s_file, size_t n_line) { glGenVertexArrays(count, arrays); GL_CHECK_BLOCK(p_s_file, n_line);  }
inline GLboolean __glachwrap_glIsVertexArray(GLuint array, const char *p_s_file, size_t n_line) { GLboolean t_result = glIsVertexArray(array); GL_CHECK_BLOCK(p_s_file, n_line); return t_result; }
// GL autocheck wrapper functions

#ifdef FIX_GL_FUNCTION_LINKING_CONFLICTS
#undef glBindVertexArray
#undef glDeleteVertexArrays
#undef glGenVertexArrays
#undef glIsVertexArray
#endif // FIX_GL_FUNCTION_LINKING_CONFLICTS
// undef function names as they need to be renamed again

#define glBindVertexArray(array) __glachwrap_glBindVertexArray(array, __FILE__, __LINE__)
#define glDeleteVertexArrays(count,arrays) __glachwrap_glDeleteVertexArrays(count, arrays, __FILE__, __LINE__)
#define glGenVertexArrays(count,arrays) __glachwrap_glGenVertexArrays(count, arrays, __FILE__, __LINE__)
#define glIsVertexArray(array) __glachwrap_glIsVertexArray(array, __FILE__, __LINE__)
// rename the originals to call the wrapper functions

#elif !defined(GLES20EMU_CPP) // GL_AUTOCHECK && (_DEBUG || FORCE_GL_CHECK) && !FORCE_AUTOCHECK_OFF_ON_GL_EXTENSIONS
// do not patch this for GLES20Emu.cpp

inline void __gles20emupatch_glBindVertexArray(GLuint array) { if(array) glBindVertexArray(array); else glBindVertexArray(__n_gles20emu_default_vertex_array); }
// some OpenGL versions do not allow VAO 0 to be used for actual rendering

#ifdef FIX_GL_FUNCTION_LINKING_CONFLICTS
#undef glBindVertexArray
#endif // FIX_GL_FUNCTION_LINKING_CONFLICTS
// undef function name as it needs to be renamed again

#define glBindVertexArray(array) __gles20emupatch_glBindVertexArray(array)
// rename the original to call the wrapper function

#endif // GL_AUTOCHECK && (_DEBUG || FORCE_GL_CHECK) && !FORCE_AUTOCHECK_OFF_ON_GL_EXTENSIONS

#endif // GL_ARB_vertex_array_object

#endif // __OPENGLES2_EMULATOR_INCLUDED
