/*
								+---------------------------------+
								|                                 |
								|   ***   Shader compiler   ***   |
								|                                 |
								|  Copyright   -tHE SWINe- 2011  |
								|                                 |
								|           Shader.cpp            |
								|                                 |
								+---------------------------------+
*/

/**
 *	@file Shader.cpp
 *	@author -tHE SWINe-
 *	@brief Shader compiler and binary cache.
 */

// OpenGL and operating system includes.
#include "GLES20Emu.h"

// Standard includes.
#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <stdexcept>
#include <vector>
#include <string>
#include <algorithm>
#include <memory>

#include "Shader.h"

/*
 *								=== TShader ===
 */

TShader::TShader() // throws(std::bad_alloc)
	:p_reference_num(new size_t[1]), n_vertex_shader(0),
	n_fragment_shader(0), n_program_object(0)
{
	*p_reference_num = 1;
}

TShader::TShader(const TShader &r_t_sh)
	:p_reference_num(r_t_sh.p_reference_num),
	n_vertex_shader(r_t_sh.n_vertex_shader),
	n_fragment_shader(r_t_sh.n_fragment_shader),
	n_program_object(r_t_sh.n_program_object)
{
	if(p_reference_num)
		++ *p_reference_num;
}

TShader &TShader::operator =(const TShader &r_t_sh)
{
	if(!p_reference_num || !(-- *p_reference_num)) {
		if(p_reference_num)
			delete[] p_reference_num;
		if(n_vertex_shader)
			glDeleteShader(n_vertex_shader);
		if(n_fragment_shader)
			glDeleteShader(n_fragment_shader);
		if(n_program_object)
			glDeleteProgram(n_program_object);
	}
	// cleanup

	p_reference_num = r_t_sh.p_reference_num,
	n_vertex_shader = r_t_sh.n_vertex_shader;
	n_fragment_shader = r_t_sh.n_fragment_shader;
	n_program_object = r_t_sh.n_program_object;

	if(p_reference_num)
		++ *p_reference_num;

	return *this;
}

TShader::~TShader()
{
	if(!p_reference_num || !(-- *p_reference_num)) {
		if(p_reference_num)
			delete[] p_reference_num;
		if(n_vertex_shader)
			glDeleteShader(n_vertex_shader);
		if(n_fragment_shader)
			glDeleteShader(n_fragment_shader);
		if(n_program_object)
			glDeleteProgram(n_program_object);
	}
}

/*
 *								=== ~TShader ===
 */

/*
 *								=== CGLShaderCompiler ===
 */

class CGLShaderCompiler::CBindAttribLocation {
protected:
	GLuint m_n_program;

public:
	inline CBindAttribLocation(GLuint n_program)
		:m_n_program(n_program)
	{}

	inline void operator ()(const std::pair<std::string, int> &r_binding) const
	{
		glBindAttribLocation(m_n_program,
			r_binding.second, r_binding.first.c_str());
	}
};

class CGLShaderCompiler::CBindFragDataLocation {
protected:
	GLuint m_n_program;

public:
	inline CBindFragDataLocation(GLuint n_program)
		:m_n_program(n_program)
	{}

	inline void operator ()(const std::pair<std::string, int> &r_binding) const
	{
		/*glBindFragDataLocation(m_n_program,
			r_binding.second, r_binding.first.c_str());*/ // OpenGL ES 2.0 seems not to have this
	}
};

bool CGLShaderCompiler::CheckShader(GLuint n_shader_object, const char *p_s_shader_name)
{
	bool b_compiled;
	{
		int n_tmp;
		glGetShaderiv(n_shader_object, GL_COMPILE_STATUS, &n_tmp);
		b_compiled = n_tmp == GL_TRUE;
	}
	int n_log_length;
	glGetShaderiv(n_shader_object, GL_INFO_LOG_LENGTH, &n_log_length);
	// query status ...

	if(n_log_length > 1) {
		char *p_s_info_log;
		if(!(p_s_info_log = new(std::nothrow) char[n_log_length + 1]))
			return false;
		// alloc temp storage for log string

		glGetShaderInfoLog(n_shader_object, n_log_length, &n_log_length, p_s_info_log);
		// get log string

#ifndef _HAVE_OPENGLES2
		//fprintf(stderr, "\nwarning: note the shader line numbers are offset by +4 !!!\n"); // not anymore
#endif // !_HAVE_OPENGLES2
		printf("GLSL compiler (%s):\n%s\n", p_s_shader_name, p_s_info_log);
		// print info log

		delete[] p_s_info_log;
		// delete temp storage
	}
	// get info-log

	return b_compiled;
}

bool CGLShaderCompiler::CompileProgram(TShader &r_t_program,
	const char *p_s_vertex_shader_src, const char *p_s_fragment_shader_src,
	const char *p_s_program_name, const char *p_s_attribute_bindings,
	bool b_verbose, bool b_check_shader, bool b_check_program)
{
	const char *p_s_vertex_shader_source = p_s_vertex_shader_src;
	const char *p_s_vertex_shader_binding = p_s_attribute_bindings;
	const char *p_s_fragment_shader_source = p_s_fragment_shader_src;
	const char *p_s_fragment_shader_binding = "";
	// shaders loaded from xml

	std::list<std::pair<std::string, int> > vertex_input_binding;
	std::list<std::pair<std::string, int> > fragment_output_binding;

	if((p_s_vertex_shader_binding && !Parse_Binding(vertex_input_binding,
	   p_s_vertex_shader_binding)) || (p_s_fragment_shader_binding &&
	   !Parse_Binding(fragment_output_binding,
	   p_s_fragment_shader_binding)))
		return false;
	// parse input/output bindings

	TShader t_shader;

	for(int n_pass = 0; n_pass < 2; ++ n_pass) {
#ifdef _HAVE_OPENGLES2
		const char *p_s_source_code = (n_pass)? p_s_fragment_shader_source :
			p_s_vertex_shader_source;
#else // _HAVE_OPENGLES2
		std::string s_source2;
		try {
			std::string s_temp_shader_source;
			do {
				int n_major, n_minor;
				const char *p_s_glsl_version = (const char*)glGetString(GL_SHADING_LANGUAGE_VERSION);
				if(!p_s_glsl_version || sscanf(p_s_glsl_version, "%d.%d", &n_major, &n_minor) != 2) {
					n_major = 0;
					n_minor = 0;
				}
				// try to obtain GLSL version

				if(CGLES20ExtensionHandler::b_Support_OpenGL(3, 3) && (n_major > 3 || (n_major == 3 && n_minor >= 30)))
					s_source2 = "#version 330\r\n"; // have to specify GLSL version on PC
				else if(n_major > 1 || (n_major == 1 && n_minor >= 30))
					s_source2 = "#version 130\r\n"; // this is fallback; some OpenGL 2.1 implementations with recent drivers do support this
				else {
					// no version specifier for very old implementations

					s_temp_shader_source = (n_pass)? p_s_fragment_shader_source : p_s_vertex_shader_source;
					// copy shader source ...

					size_t n_pos = 0;
					while((n_pos = s_temp_shader_source.find("precision", n_pos)) != std::string::npos) {
						size_t b = n_pos, e = s_temp_shader_source.length();
						b += strlen("precision");
						// default precision specification statement

						if(b == e) {
							++ n_pos; // unknown syntax, just continue
							continue;
						}
						while(b < e && isspace(s_temp_shader_source[b]))
							++ b;
						// skip space

						if(b + 4 < e && !memcmp(s_temp_shader_source.data() + b, "lowp", 4 * sizeof(char)))
							b += 4;
						else if(b + 7 < e && !memcmp(s_temp_shader_source.data() + b, "mediump", 7 * sizeof(char)))
							b += 7;
						else if(b + 5 < e && !memcmp(s_temp_shader_source.data() + b, "highp", 5 * sizeof(char)))
							b += 5;
						else {
							++ n_pos; // unknown syntax, just continue
							continue;
						}
						// skip precission qualifier

						if(b == e) {
							++ n_pos; // unknown syntax, just continue
							continue;
						}
						while(b < e && isspace(s_temp_shader_source[b]))
							++ b;
						// skip space

						if(b == e) {
							++ n_pos; // unknown syntax, just continue
							continue;
						}
						while(b < e && isalnum(s_temp_shader_source[b]))
							++ b;
						// skip data type (eg. float, vec4, ...)

						while(b < e && isspace(s_temp_shader_source[b]))
							++ b;
						// skip space

						if(b == e || s_temp_shader_source[b] != ';') {
							++ n_pos; // unknown syntax, just continue
							continue;
						}
						++ b; // skip ';'
						// skip semicolon

						int n_begin = n_pos;
						int n_end = b;

						s_temp_shader_source.erase(n_begin, n_end - n_begin);
						// erase the default precission specification
					}
					// this erases any default precision qualifiers there are

					// todo - it would also be nice to erase precision qualifiers from
					// the individual variables (eg. "uniform highp float time;")

					if(n_pass)
						p_s_fragment_shader_source = s_temp_shader_source.c_str();
					else
						p_s_vertex_shader_source = s_temp_shader_source.c_str();
					// just need to get rid of "precision highp float;" lines because these compilers do not understand those
					break;
				}
				// set shading language version

				if(n_pass) {
					// fragment
					s_source2 += "out vec4 _v_gl_frag_color_;\r\n";
					s_source2 += "#define gl_FragColor _v_gl_frag_color_\r\n";
					s_source2 += "#define varying in\r\n";
					s_source2 += "#line 0\r\n"; // fix line numbering ...
				} else {
					// vertex
					s_source2 += "#define attribute in\r\n";
					s_source2 += "#define varying out\r\n";
					s_source2 += "#line 0\r\n"; // fix line numbering ...
				};
				// add directives, if required
			} while(0);
			// add some directives for the OpenGL ES shaders to work on OpenGL hardware
			// todo - intercept glShaderSource and preprocess the source code in there

			s_source2 += (n_pass)? p_s_fragment_shader_source : p_s_vertex_shader_source;
			// include the main shader source
		} catch(std::bad_alloc&) {
			return false;
		}
		const char *p_s_source_code = s_source2.c_str();
#endif // _HAVE_OPENGLES2

		GLuint n_shader = glCreateShader((n_pass)? GL_FRAGMENT_SHADER : GL_VERTEX_SHADER);
		glShaderSource(n_shader, 1, &p_s_source_code, NULL);
		glCompileShader(n_shader);
		// compile the shader

		std::string s_shader_name;
		stl_ut::Format(s_shader_name, "%s::%s", p_s_program_name, (n_pass)? "fragment" : "vertex");
		if(b_check_shader && !CheckShader(n_shader, s_shader_name.c_str())) {
			glDeleteShader(n_shader);
			return false;
		}
		// check for compile-time errors

		if(n_pass)
			t_shader.n_fragment_shader = n_shader;
		else
			t_shader.n_vertex_shader = n_shader;
		// put shaders to program object
	}

	t_shader.n_program_object = glCreateProgram();
	glAttachShader(t_shader.n_program_object, t_shader.n_vertex_shader);
	glAttachShader(t_shader.n_program_object, t_shader.n_fragment_shader);
	// create program object

	std::for_each(vertex_input_binding.begin(),
		vertex_input_binding.end(),
		CBindAttribLocation(t_shader.n_program_object));
	// binds vertex attribute locations

	std::for_each(fragment_output_binding.begin(),
		fragment_output_binding.end(),
		CBindFragDataLocation(t_shader.n_program_object));
	// bids fragment data location

	glLinkProgram(t_shader.n_program_object);
	if(b_check_program && !CheckProgram(
	   t_shader.n_program_object, p_s_program_name))
		return false;
	// links program

	r_t_program = t_shader;

	return true;
}

bool CGLShaderCompiler::Parse_Binding(
	std::list<std::pair<std::string, int> > &r_binding,
	const char *p_s_options)
{
	r_binding.clear();
	// ...

	for(size_t b = 0, e = strlen(p_s_options); b != e;) {
		while(b != e && isspace(p_s_options[b]))
			++ b;
		if(b == e)
			break;
		// skip whitespace

		int n_key_begin = b;
		while(b != e && !isspace(p_s_options[b]) && p_s_options[b] != ':')
			++ b;
		int n_key_end = b;
		// scan key

		while(b != e && isspace(p_s_options[b]))
			++ b;
		// skip whitespace

		if(b == e || p_s_options[b] != ':')
			return false;
		++ b;
		// there must be a ':'

		while(b != e && isspace(p_s_options[b]))
			++ b;
		// skip whitespace

		if(b == e)
			return false;
		// there must be value

		int n_value_begin = b;
		while(b != e && !isspace(p_s_options[b]) && p_s_options[b] != ';')
			++ b;
		int n_value_end = b;
		// scan value

		while(b != e && isspace(p_s_options[b]))
			++ b;
		// skip whitespace

		if(b == e || p_s_options[b] != ';')
			return false;
		++ b;
		// there must be a ';'

		try {
			std::string s_key, s_value;
			s_key.insert(s_key.begin(), p_s_options + n_key_begin, p_s_options + n_key_end);
			s_value.insert(s_value.begin(), p_s_options + n_value_begin, p_s_options + n_value_end);
			int n_value;
			if(sscanf(s_value.c_str(), "%d", &n_value) != 1)
				return false;
			r_binding.push_back(std::make_pair(s_key, n_value));
		} catch(std::bad_alloc&) {
			return false;
		}
		// add key/value pair to the list ...
	}
	// parse the options string

	return true;
}
/*
bool CGLShaderCompiler::Load_CompiledShader(GLuint &r_n_shader, const char *p_s_shader_name,
	const char *p_s_database_name, bool b_verbose, bool b_check_shader)
{
	CXML_EventParser parser(p_s_database_name);
	CXMLNode *p_doc;
	if(!(p_doc = CXMLNode::p_Parse(parser))) {
		const char *p_s_error = "xml parse error";
		switch(parser.n_State()) {
		case CXML_EventParser::state_BadEncoding:
			p_s_error = "bad encoding";
			break;
		case CXML_EventParser::state_CodeError:
			p_s_error = "bad code";
			break;
		case CXML_EventParser::state_LexerError:
		case CXML_EventParser::state_LexerInitError:
		case CXML_EventParser::state_LexicalError:
			p_s_error = "lexical error";
			break;
		case CXML_EventParser::state_MalformedDoc:
			p_s_error = "malformed document";
			break;
		case CXML_EventParser::state_NoMemory:
			p_s_error = "low memory";
			break;
		case CXML_EventParser::state_UnexpectedEOF:
			p_s_error = "unexpected end of file";
			break;
		case CXML_EventParser::state_UnexpectedToken:
			p_s_error = "unexpected token";
			break;
		case CXML_EventParser::state_UnsupportedNode:
			p_s_error = "unsupported node";
			break;
		}
		fprintf(stderr, "error: file \'%s\', line %d, column %d: xml parse error\n",
			p_s_database_name, parser.n_Cur_Line() + 1, parser.n_Cur_Column() + 1);
		return false;
	}
	std::auto_ptr<CXMLNode> doc(p_doc); // just delete me when it's over
	p_doc = (CXMLNode*)p_doc->p_FindChild("source"); // everything is in <source>
	// parse config xml

	const char *p_s_vendor = (const char*)glGetString(GL_VENDOR);
	const char *p_s_renderer = (const char*)glGetString(GL_RENDERER);
	// get vendor / renderer

	for(const CXMLNode *p_shader =
	   p_doc->p_FindChild("shader"); p_shader->b_IsElem();
	   p_shader = p_doc->p_FindNextChild(p_shader, "shader")) {
		const std::string *p_name = p_shader->p_Attrib("name");
		const std::string *p_type = p_shader->p_Attrib("type");
		if(!p_name || p_name->compare(p_s_shader_name) || !p_type)
			continue;
		// find such xml node, which has the same name as p_s_shader_name

		if(p_shader->n_Child_Num() < 1)
			continue;
		const std::string &s_source = p_shader->p_Child(0)->p_ToCData()->s_Value();
		if(s_source.empty())
			continue;
		// get shadrer source code

		TSHA1 t_hash;
		{
			CStreamHash<TSHA1> hasher;
			hasher.Process_Data(p_s_vendor, strlen(p_s_vendor) * sizeof(char));
			hasher.Process_Data(p_s_renderer, strlen(p_s_renderer) * sizeof(char));
			hasher.Process_Data(s_source.data(), s_source.length() * sizeof(char));
			t_hash = hasher.t_Result();
		}
		// hash source code and environment parameters

		// @todo - find compiled shader binary

		if(b_verbose) {
			printf("compiling shader \'%s#%s\' (0x%08x%08x%08x%08x%08x)\n",
				p_s_database_name, p_s_shader_name,
				htonl(t_hash[0]), htonl(t_hash[1]),
				htonl(t_hash[2]), htonl(t_hash[3]), htonl(t_hash[4])); // note this is supposed to be big endian
		}
		// verbose

#ifdef _TEGRA
		const char *p_s_source_code = s_source.c_str();
#else //_TEGRA
		std::string s_source2;
		try {
			s_source2 = "#version 330\r\n"; // have to specify GLSL version on PC
			s_source2 += s_source;
		} catch(std::bad_alloc&) {
			return false;
		}
		const char *p_s_source_code = s_source2.c_str();
#endif //_TEGRA

		//printf("DEBUG: \'%s#%s\':\n%s\n", p_s_database_name,
		//	p_s_shader_name, p_s_source_code); // debug

		GLuint n_shader = glCreateShader((p_type->compare("vertex"))? GL_FRAGMENT_SHADER : GL_VERTEX_SHADER);
		glShaderSource(n_shader, 1, &p_s_source_code, NULL);
		glCompileShader(n_shader);
		// compile the shader

		if(b_check_shader && !CheckShader(n_shader, p_s_shader_name)) {
			glDeleteShader(n_shader);
			return false;
		}
		// check for compile-time errors

		r_n_shader = n_shader;
		// output the shader

		// @todo - save shader binary to the cache

		return true;
	}

	return false;
}*/

bool CGLShaderCompiler::CheckProgram(GLuint n_program_object, const char *p_s_program_name)
{
	bool b_linked;
	{
		int n_tmp;
		glGetProgramiv(n_program_object, GL_LINK_STATUS, &n_tmp);
		b_linked = n_tmp == GL_TRUE;
	}
	int n_length;
	glGetProgramiv(n_program_object, GL_INFO_LOG_LENGTH, &n_length);
	// query status ...

	if(n_length > 1) {
		char *p_s_info_log;
		if(!(p_s_info_log = new(std::nothrow) char[n_length + 1]))
			return false;
		// alloc temp log

		glGetProgramInfoLog(n_program_object, n_length, &n_length, p_s_info_log);
		// get info log

		printf("GLSL linker (%s):\n%s\n", p_s_program_name, p_s_info_log);
		// print info log

		delete[] p_s_info_log;
		// release temp log
	}
	// get info-log

	return b_linked;
}

/*
 *								=== ~CGLShaderCompiler ===
 */
