/*
								+---------------------------------+
								|                                 |
								|   ***   Shader compiler   ***   |
								|                                 |
								|  Copyright   -tHE SWINe- 2011  |
								|                                 |
								|            Shader.h             |
								|                                 |
								+---------------------------------+
*/

/**
 *	@file Shader.h
 *	@author -tHE SWINe-
 *	@brief Shader compiler and binary cache.
 *	@note This is not caching anything right now (@todo).
 */

#ifndef __CACHING_SHADER_COMPILER_INCLUDED
#define __CACHING_SHADER_COMPILER_INCLUDED

#include "GLES20Emu.h"
#include <list>

/**
 *	@brief generic reference-counted shader structure
 */
struct TShader {
protected:
	size_t *p_reference_num; /**< @brief reference counter */

public:
	GLuint n_vertex_shader; /**< @brief vertex shader OpenGL id */
	GLuint n_fragment_shader; /**< @brief fragment shader OpenGL id */
	GLuint n_program_object; /**< @brief program object OpenGL id */

	/**
	 *	@brief default constructor
	 */
	TShader();

	/**
	 *	@brief copy constructor
	 *	@param[in] r_t_sh is hsader to be copied (reference counter is increased)
	 */
	TShader(const TShader &r_t_sh);

	/**
	 *	@brief destructor (decreases reference counter
	 *		and deletes OpenGL objects, if null)
	 */
	~TShader();

	/**
	 *	@brief copy operator
	 *	@param[in] r_t_sh is hsader to be copied (reference counter is increased)
	 *	@return Returns reference to this.
	 */
	TShader &operator =(const TShader &r_t_sh);

	/**
	 *	@brief makes shader active
	 *	@note After calling this function, one usually
	 *		calls glUniform*() to set shader parameters.
	 */
	inline void Bind()
	{
		glUseProgram(n_program_object);
	}
};
                                                                                                    
/**
 *	@brief utility for easy and fast compiling of shaders
 */
class CGLShaderCompiler {
protected:
	class CBindAttribLocation;
	class CBindFragDataLocation;

public:
	/**
	 *	@brief loads compiled shaders from cache (or compiles them
	 *		in case it was not present) and link them to a program
	 *
	 *	@param[out] r_t_program is TShader structure to be loaded (output of the function)
	 *	@param[in] p_s_vertex_shader_src is 
	 *	@param[in] p_s_fragment_shader_src is 
	 *	@param[in] p_s_program_name is program name (only used for error messages)
	 *	@param[in] p_s_attribute_bindings is 
	 *	@param[in] b_verbose is verbosity flag (prints whether the program was cached or not)
	 *	@param[in] b_check_shader is compile error checking flag (deletes
	 *		shader on errors, prints them to stderr)
	 *	@param[in] b_check_program is link error checking flag (deletes
	 *		program on errors, prints them to stderr)
	 *
	 *	@return Returns true on success, false on failure.
	 */
	static bool CompileProgram(TShader &r_t_program,
		const char *p_s_vertex_shader_src, const char *p_s_fragment_shader_src,
		const char *p_s_program_name, const char *p_s_attribute_bindings,
		bool b_verbose = true, bool b_check_shader = true,
		bool b_check_program = true);

	/**
	 *	@brief Checks shader compile state, prints error(s) (if any) to stderr.
	 *
	 *	@param[in] n_shader_object is shader object id
	 *	@param[in] p_s_shader_name is shader object name (to be displayed to the user)
	 *
	 *	@return Returns true on success, false on failure.
	 */
	static bool CheckShader(GLuint n_shader_object, const char *p_s_shader_name);

	/**
	 *	@brief Checks program link state, prints error(s) (if any) to stderr.
	 *
	 *	@param[in] n_program_object is program object id
	 *	@param[in] p_s_program_name is program object name (to be displayed to the user)
	 *
	 *	@return Returns true on success, false on failure.
	 */
	static bool CheckProgram(GLuint n_program_object, const char *p_s_program_name);

protected:
	static bool Parse_Binding(std::list<std::pair<std::string, int> > &r_binding,
		const char *p_s_options);
};

#endif //__CACHING_SHADER_COMPILER_INCLUDED
